﻿-------------------------------------------------------------------------------

function nUI:battlefieldMapFix()

	local wmfInit = function()
		this = WorldMapFrame;
		WorldMapFrame_OnLoad();
	end;
	
	wmfInit();
	
	if not IsAddOnLoaded( "Blizzard_BattlefieldMinimap" ) then 
	
		LoadAddOn( "Blizzard_BattlefieldMinimap" );
		
	end

	if IsAddOnLoaded( "Blizzard_BattlefieldMinimap" ) then 

		hooksecurefunc( "BattlefieldMinimap_Toggle",   function() nUI:onBMMClick(); end );

		BattlefieldMinimap:Show();
		BattlefieldMinimap:SetParent( WorldFrame );
		BattlefieldMinimap:SetFrameStrata( "BACKGROUND" );
		BattlefieldMinimap:SetFrameLevel( 4 );
		BattlefieldMinimap:EnableMouse( true );
		BattlefieldMinimapCloseButton:Hide();
		BattlefieldMinimapCorner:Hide();
		BattlefieldMinimapBackground:Hide();
		BattlefieldMinimap_SetOpacity( 0.45 );
		
		nUI:bmmShow();
		
		SHOW_BATTLEFIELD_MINIMAP = 1;

		-- this little hack effectively prevents the user from seeing or
		-- accessing that annoying battlefield minimap tab
		
		BattlefieldMinimap.oldOnUpdate = BattlefieldMinimap:GetScript( "OnUpdate" );	

		BattlefieldMinimap:SetScript( "OnUpdate", 
			function()	
				
				if BattlefieldMinimap.oldOnUpdate then
					BattlefieldMinimap:oldOnUpdate();
				end
				
				if BattlefieldMinimapTab:IsShown() then
					BattlefieldMinimapTab:Hide();
				end
				
				if nUI.bmm_frame.mapOpen then
					RequestBattlefieldPositions();
					WorldMapFrame_Update();
					WorldMapButton_OnUpdate(arg1);
				end
					
			end		
		);

		-- size the map to fit the info frame
		
		BattlefieldMinimap:SetScale( (nUI.info_frame:GetHeight() - 8) / BattlefieldMinimap:GetHeight() * 1.05 );

		-- disable the hideaway function
		
		BattlefieldMinimap:SetScript( "OnHide", function() BattlefieldMinimap:Show(); end );
		
		-- add the ability to click the battlefield minimap
		
		nUI.bmm_frame = CreateFrame( "Button", "nUI_BMM_Button", BattlefieldMinimap );
		
		nUI.bmm_frame:SetAllPoints( BattlefieldMinimap );
		nUI.bmm_frame:SetScript( "OnClick", function() self:onBMMClick(); end );
		
		-- hook the world map's OnShow so if the player tries to open the map while
		-- we've stolen the detail frame, we can put it back
		
		nUI.bmm_frame.wmfOnShow = WorldMapFrame:GetScript( "OnShow" );
		
		WorldMapFrame:SetScript( "OnShow",
			function()
				if nUI.bmm_frame.mapOpen then nUI:onBMMClick(); end
				if nUI.bmm_frame.wmfOnShow then nUI.bmm_frame:wmfOnShow(); end
			end
		);
					
	end
end

-------------------------------------------------------------------------------

function nUI:onBMMClick()

	if not nUI.bmm_frame.mapOpen and not InCombatLockdown() then

		-- we do this so that mods which hook the World Map OnShow() or
		-- the WorldMap onUpdate functions will do their thing and update
		-- the map accordingly.
		
		WorldMapFrame:Show();
		WorldMapFrame:Hide();
		
		nUI.bmm_frame.wmdScale  = WorldMapDetailFrame:GetScale();
		nUI.bmm_frame.wmdParent = WorldMapDetailFrame:GetParent();
		nUI.bmm_frame.wmdStrata = WorldMapDetailFrame:GetFrameStrata();
		nUI.bmm_frame.wmbParent = WorldMapButton:GetParent();
		nUI.bmm_frame.mapOpen   = true;

		SetMapToCurrentZone();
		PlaySound("igQuestLogOpen");
		CloseDropDownMenus();
		
		WorldMapButton:SetParent( WorldMapDetailFrame );
		WorldMapButton:Show();
		WorldMapDetailFrame:ClearAllPoints();
		WorldMapDetailFrame:SetFrameStrata( "HIGH" );
		WorldMapDetailFrame:SetParent( WorldFrame );
		WorldMapDetailFrame:SetPoint( "TOP", nUI_Console, "BOTTOM", 0, -20 );
		WorldMapDetailFrame:SetPoint( "BOTTOM", nUI_Dashboard, "TOP", 0, 20 );
		WorldMapDetailFrame:SetScale( (WorldMapDetailFrame:GetTop() - WorldMapDetailFrame:GetBottom()) / WorldMapDetailFrame:GetHeight() );
		WorldMapDetailFrame:Show();
		
		UpdateMicroButtons();
		SetMapToCurrentZone();
		PlaySound("igQuestLogOpen");
		CloseDropDownMenus();
		RequestBattlefieldPositions();
		WorldMapFrame_PingPlayerPosition();

		WorldMapDetailFrame:SetScript( "OnUpdate",
			function()
				this = WorldMapFrame;
				WorldMapFrame_Update();
				RequestBattlefieldPositions();
				
				if WorldMapPlayerArrow and WorldMapPlayerArrow:GetParent() ~= WorldMapDetailFrame then
					WorldMapPlayerArrow:SetParent( WorldMapDetailFrame );
				end
				
				if PlayerArrowEffectFrame and PlayerArrowEffectFrame:GetParent() ~= WorldMapDetailFrame then
					PlayerArrowEffectFrame:SetParent( WorldMapDetailFrame );
				end
			end
		);

		WorldMapDetailFrame:SetScript( "OnKeyDown",
			function()
				local binding = GetBindingFromClick(arg1)
				if ((binding == "TOGGLEWORLDMAP") or (arg1 == "ESCAPE")) then
					RunBinding("TOGGLEWORLDMAP");
				elseif ( binding == "SCREENSHOT" ) then
					RunBinding("SCREENSHOT");
				elseif ( binding == "MOVIE_RECORDING_STARTSTOP" ) then
					RunBinding("MOVIE_RECORDING_STARTSTOP");
				end
			end		
		);

	elseif nUI.bmm_frame.mapOpen then

		nUI.bmm_frame.mapOpen = false;

		WorldMapDetailFrame:SetScript( "OnUpdate", nil );
		WorldMapDetailFrame:SetScript( "OnKeyDown", nil );
		
		WorldMapDetailFrame:SetParent( nUI.bmm_frame.wmdParent );
		WorldMapDetailFrame:SetScale( nUI.bmm_frame.wmdScale );
		WorldMapDetailFrame:SetFrameStrata( nUI.bmm_frame.wmdStrata );
		WorldMapDetailFrame:ClearAllPoints();
		WorldMapDetailFrame:SetPoint( "TOPLEFT", "WorldMapPositioningGuide", "TOP", -502, -69 );
		WorldMapDetailFrame:SetAlpha( 1 );
		WorldMapDetailFrame:Show();
		WorldMapButton:SetParent( nUI.bmm_frame.wmbParent );
		WorldMapButton:Show();

	end	

end

-------------------------------------------------------------------------------

function nUI:bmmHide()
	
	if nUI.info_frame.bmm_visible then
		BattlefieldMinimap:SetPoint( "TOPLEFT", UIParent, "BOTTOM", 0, -1000 );
		nUI.info_frame.bmm_visible = false;
	end
end

-------------------------------------------------------------------------------

function nUI:bmmShow()
	
	if not nUI.info_frame.bmm_visible then		
		BattlefieldMinimap:SetPoint( "TOPLEFT", nUI.info_frame, "TOPLEFT", 1, -1 );
		nUI.info_frame.bmm_visible = true;
	end
end