﻿-------------------------------------------------------------------------------
-- local cache

local UnitIsUnit                = UnitIsUnit;
local GetPlayerBuff             = GetPlayerBuff;
local GetPlayerBuffName         = GetPlayerBuffName;
local GetPlayerBuffApplications = GetPlayerBuffApplications;
local GetPlayerBuffTexture      = GetPlayerBuffTexture;
local GetPlayerBuffTimeLeft     = GetPlayerBuffTimeLeft;
local GetPlayerBuffDispelType   = GetPlayerBuffDispelType;
local UnitBuff                  = UnitBuff;
local UnitDebuff                = UnitDebuff;
local UnitExists                = UnitExists;
local UnitIsDeadOrGhost         = UnitIsDeadOrGhost;
local GetTime                   = GetTime;
local UIFrameFlashRemoveFrame   = UIFrameFlashRemoveFrame;
local UIFrameFlash              = UIFrameFlash;
local CooldownFrame_SetTimer    = CooldownFrame_SetTimer;
local nUI_DurationText          = nUI_DurationText;

-------------------------------------------------------------------------------

function nUI_SortBuffs( left, right )

	-- we place buffs and debuffs with a duration ahead of those without one
	
	if not left.duration then return false;
	elseif left.duration == 0 then return false;
	elseif not right.duration then return true;
	elseif right.duration == 0 then return true;
	
	-- otherwise whichever has the lowest duration will go first
	
	else return left.duration < right.duration;
	end
	
end

-------------------------------------------------------------------------------

nUI.Class.Buffs = nUI.Class:CreatePrototype( nUI.Class.StatusBar );
local prototype = nUI.Class.Buffs.prototype;

-------------------------------------------------------------------------------

function prototype:init( parent, id, layout, unit_id )
	
	self.parent      = parent;
	self.id          = id;
	self.rid         = 1;
	self.unit_id     = unit_id;
	self.layout      = layout;
	self.guid_timer  = 0;
	self.flash_timer = 0;
	self.flash_val   = 0;
	self.num_buffs   = 0;
	self.num_debuffs = 0;
	self.Buffs       = {};
	self.Debuffs     = {};
		
	self:layoutFrame();
	self:registerEvents();
	
end

-------------------------------------------------------------------------------

function prototype:layoutFrame()

	self.event_frame = CreateFrame( "Frame", self.layout.name.."_Events", self.parent );
	self.buff_frame  = CreateFrame( "Frame", self.layout.name.."_Frame", self.parent );
	
	self.buff_frame.texture = self.buff_frame:CreateTexture();
	self.buff_frame.texture:SetAllPoints( self.buff_frame );
--	self.buff_frame.texture:SetTexture( 0, 0, 0, 0.5 );
	
end

-------------------------------------------------------------------------------

function prototype:registerEvents()
	
	self.event_frame:SetScript( "OnEvent", function() self:onEvent(); end );
	self.event_frame:SetScript( "OnUpdate", function( who, elapsed ) self:onUpdate( elapsed ); end );
	
	self.event_frame:RegisterEvent( "PLAYER_AURAS_CHANGED" );
	self.event_frame:RegisterEvent( "UNIT_AURA" );
	
end

-------------------------------------------------------------------------------

function prototype:onUpdate( elapsed )

	self.guid_timer = self.guid_timer + elapsed;

	-- check to see if the thing this unit refers to has changed
	
	if self.guid_timer >= 0.08 then -- check at 12.5fps

		self.guid_timer = 0;
		
		local unit_id = self.unit_id;
		local guid    = UnitGUID( unit_id );

		if not UnitExists( unit_id ) or UnitIsDeadOrGhost( unit_id ) then

			if self.buff_frame:IsShown() then

				self:Hide();
				self:scanBuffs( unit_id );

			end
			
		elseif self.guid ~= UnitGUID( self.unit_id ) then

			self:Show();
			self.guid = guid;
			self:scanBuffs( unit_id );
			
		elseif not self.buff_frame:IsShown() then
			
			self:Show();
			
		end
	end

	-- if we have any buffs flashing, make then all fade in and out together
	
	if self.layout.visible ~= "no" then
				
		if self.layout.flash ~= 0 then
			
			self.flash_timer = self.flash_timer + elapsed;
			self.flash_val   = self.flash_val + elapsed;
			
			if self.flash_timer >= 0.0167 then -- roughly 60fps for smooth animation
			
				local flash_rate = self.layout.flash_rate;
				local alpha = self.flash_val % flash_rate;
				
				if alpha < flash_rate * 0.25 then alpha = alpha * 4;
				elseif alpha < flash_rate * 0.75 then alpha = 1;
				else alpha = (flash_rate - alpha) * 4;
				end
				
				for i in pairs( self.Buffs ) do
					
					if self.Buffs[i].is_flashing then
						self.Buffs[i].frame:SetAlpha( alpha );
					end				
				end
				
				for i in pairs( self.Debuffs ) do
					
					if self.Debuffs[i].is_flashing then
						self.Debuffs[i].frame:SetAlpha( alpha );
					end				
				end
			end
		end
	end
end

-------------------------------------------------------------------------------

function prototype:onEvent()
	
	if event == "PLAYER_AURAS_CHANGED" then

		self:scanBuffs( "player" );
	
	elseif event == "UNIT_AURA" then

		self:scanBuffs( arg1 );
		
	end
end

-------------------------------------------------------------------------------

function prototype:scanBuffs( unit_id )
	
	if UnitIsUnit( self.unit_id, unit_id ) then
			
		-- if we have existing frames, clear them
		
		for i in pairs( self.Buffs ) do
			if self.Buffs[i].frame then
				self.Buffs[i].frame:ClearAllPoints();
				self.Buffs[i].frame.duration:SetScript( "OnUpdate", nil );
				self.Buffs[i].frame:Hide();
			end
		end
	
		self.Buffs   = {};
		self.num_buffs = 0;
				
		for i in pairs( self.Debuffs ) do
			if self.Debuffs[i].frame then
				self.Debuffs[i].frame.duration:SetScript( "OnUpdate", nil );
				self.Debuffs[i].frame:Hide();
			end
		end
				
		self.Debuffs = {};
		self.num_debuffs = 0;
	
		-- scan for new buffs
	
		if UnitExists( unit_id ) and not UnitIsDeadOrGhost( unit_id ) then
			
			local type = self.layout.type;
			
			if type == "adapt" then 
				type = UnitCanAttack( "player", unit_id ) and "buff" or "debuff"; 
			end
			
			if type == "adapt-" then 
				type = UnitCanAttack( "player", unit_id ) and "debuff" or "buff"; 
			end
	
			if type ~= "debuff" then self:updateBuffs( unit_id ); end
			if type ~= "buff" then self:updateDebuffs( unit_id ); end
		
			self:updateDisplay();
		end
	end
end

-------------------------------------------------------------------------------

function prototype:updateBuffs( unit_id )

	if UnitIsUnit( self.unit_id, unit_id ) then

		local is_player = UnitIsUnit( unit_id, "player" );
		local castable  = self.layout.type == "casts";
		
		-- spin the list of possible buffs and see what's there
		
		for i=1, 40 do

			-- check for a buff on this index
			
			local buff = {};
			
			buff.id           = i;
			buff.now          = GetTime();
			buff.until_cancel = 0;
			
			buff.name, buff.rank, buff.icon, buff.count, 
			buff.duration, buff.remains = UnitBuff( unit_id, i );

			if buff.name and (not castable or (castable and buff.duration)) then
								
				if is_player then
					buff.type = GetPlayerBuffDispelType( buff.name, buff.rank );
				end
				
				self.Buffs[buff.name.." "..buff.rank] = buff;
				self.num_buffs = self.num_buffs+1;

			end
		end
		
		-- if this is the player, see what can be cancelled
		
		if is_player then

			for i=1,40 do
				
				local index, until_cancel = GetPlayerBuff( i, "HELPFUL|PASSIVE" );
				
				if index > 0 then
					
					local name, rank = GetPlayerBuffName( index );
					local buff = self.Buffs[name.." "..rank];
										
					if buff then 
						buff.until_cancel = until_cancel; 
						buff.idx          = index;
					end					
				end
			end
		end							
					
		table.sort( self.Buffs );
		
	end
end

-------------------------------------------------------------------------------

function prototype:updateDebuffs( unit_id )

	if UnitIsUnit( self.unit_id, unit_id ) then

		local is_player = UnitIsUnit( unit_id, "player" );
		local castable  = self.layout.type == "casts";
		
		-- spin the list of possible debuffs and see what's there
		
		for i=1, 40 do

			-- check for a debuff on this index
			
			local debuff = {};
			
			debuff.id           = i;
			debuff.harmful      = true;
			debuff.now          = GetTime();
			debuff.until_cancel = 0;
			
			debuff.name, debuff.rank, debuff.icon, debuff.count, 
			debuff.type, debuff.duration, debuff.remains = UnitDebuff( unit_id, i );

			if debuff.name and (not castable or (castable and debuff.duration)) then
				
				if is_player then
					debuff.type = GetPlayerBuffDispelType( debuff.name, debuff.rank );
				end
								
				self.Debuffs[debuff.name.." "..debuff.rank] = debuff;
				self.num_debuffs = self.num_debuffs+1;

			end			
		end
		
		-- if this is the player, see what can be cancelled
		
		if is_player then

			for i=1,40 do
				
				local index, until_cancel = GetPlayerBuff( i, "HARMFUL" );
				
				if index > 0 then
					
					local name, rank = GetPlayerBuffName( index );
					local debuff = self.Debuffs[name.." "..rank];
										
					if debuff then 
						debuff.until_cancel = until_cancel; 
						debuff.idx          = index;
					end					
				end
			end
		end							
		
		table.sort( self.Debuffs );
		
	end
end

-------------------------------------------------------------------------------

function prototype:updateDisplay()	

	if self.layout.visible == "yes" then
			
		local frames    = {};
		local idx       = 1;
		local color_set = {};
				
		color_set["Debuff"]  = self.layout.Colors.Debuff;
		color_set["Magic"]   = self.layout.Colors.Magic;
		color_set["Poison"]  = self.layout.Colors.Poison;
		color_set["Curse"]   = self.layout.Colors.Curse;
		color_set["Disease"] = self.layout.Colors.Disease;	
	
		for i in pairs( self.Buffs ) do
		
			frames[idx] = self:createBuffFrame( self.Buffs[i], idx, color_set );
			
			if frames[idx] then 
				idx = idx+1; 
			end		
		end
	
		for i in pairs( self.Debuffs ) do
		
			frames[idx] = self:createBuffFrame( self.Debuffs[i], idx, color_set );
			
			if frames[idx] then 
				idx = idx+1; 
			end
		end
	
		-- the fun part... laying it all out...
		
		local orient = self.layout.Shape.orient;
		local grow   = self.layout.Shape.grow;
		local size   = self.layout.Shape.size;
		local rows   = self.layout.Shape.rows;
		local cols   = math.ceil( 40 / rows );
		local gap    = self.layout.Shape.gap;

		local last_col, next_row;
		local col_hgap, col_vgap;
		local row_hgap, row_vgap;
		
		if orient == "BOTTOMRIGHT" then			
			if grow == "horizontal" then
				col_hgap = -gap;
				col_vgap = 0;
				row_hgap = 0;
				row_vgap = gap;
				last_col = "BOTTOMLEFT";
				next_row = "TOPRIGHT";
			else
				col_hgap = 0;
				col_vgap = gap;
				row_hgap = -gap;
				row_vgap = 0;
				last_col = "TOPRIGHT";
				next_row = "BOTTOMLEFT";
			end
		elseif orient == "BOTTOMLEFT" then
			if grow == "horizontal" then
				col_hgap = gap;
				col_vgap = 0;
				row_hgap = 0;
				row_vgap = gap;
				last_col = "BOTTOMRIGHT";
				next_row = "TOPLEFT";
			else
				col_hgap = 0;
				col_vgap = gap;
				row_hgap = gap;
				row_vgap = 0;
				last_col = "TOPLEFT";
				next_row = "BOTTOMRIGHT";
			end
		elseif orient == "TOPRIGHT" then
			if grow == "horizontal" then
				col_hgap = -gap;
				col_vgap = 0;
				row_hgap = 0;
				row_vgap = -gap;
				last_col = "TOPLEFT";
				next_row = "BOTTOMRIGHT";
			else
				col_hgap = 0;
				col_vgap = -gap;
				row_hgap = -gap;
				row_vgap = 0;
				last_col = "BOTTOMRIGHT";
				next_row = "TOPLEFT";
			end
		else -- default is "TOPLEFT"
			if grow == "horizontal" then
				col_hgap = gap;
				col_vgap = 0;
				row_hgap = 0;
				row_vgap = -gap;
				last_col = "TOPRIGHT";
				next_row = "BOTTOMLEFT";
			else
				col_hgap = 0;
				col_vgap = -gap;
				row_hgap = gap;
				row_vgap = 0;
				last_col = "BOTTOMLEFT";
				next_row = "TOPRIGHT";
			end
		end

		if rows == 1 then cols = self:iconCount(); end
		
		if grow == "horizontal" then
			
			self.buff_frame:SetHeight( size * rows + (rows-1) * gap );
			self.buff_frame:SetWidth( size * cols + (cols-1) * gap );
		
		else
			
			self.buff_frame:SetHeight( size * cols );
			self.buff_frame:SetWidth( size * rows );
		
		end
		
		local row_start = nil;
		local last_icon = nil;
		local stop      = false;
		
		for i=1,rows do
			
			for j = 1,cols do
				
				local frame = frames[(i-1)*cols+j];
				
				if not frame then 
					stop = true;
					break; 
				end

				frame:ClearAllPoints();
				frame:SetAlpha( 1 );
				frame:Show();
				
				if i==1 and j==1 then
					frame:SetPoint( orient, self.buff_frame, orient, 0, 0 );
				elseif j==1 then
					frame:SetPoint( orient, row_start, next_row, row_hgap, row_vgap );
				else
					frame:SetPoint( orient, last_icon, last_col, col_hgap, col_vgap );
				end
							
				last_icon = frame;
				
				if j == 1 then row_start = frame; end
			end
			
			if stop then break; end
			
		end
	end
end
	
-------------------------------------------------------------------------------

function prototype:createBuffFrame( buff, index, colors )

	local frame   = nil;
	local unit_id = self.unit_id;
	local anchor  = "ANCHOR_";
	
	if buff.icon then
			
		local name   = self.layout.name.."_Icon"..index;
		local orient = self.layout.Shape.orient;
		
		anchor = anchor..(orient:match( "TOP." ) and "BOTTOM" or "TOP");
		anchor = anchor..(orient:match( ".LEFT" ) and "RIGHT" or "LEFT");
		
		-- attempt to reuse frames where we can
		
		frame = _G[name];
		
		-- otherwise, create a new frame
		
		if not frame then
				
			local color = self.layout.Colors.Remains;
			frame       = CreateFrame( "Button", name, self.buff_frame, "TargetBuffButtonTemplate" );

			frame:SetHeight( self.layout.Shape.size );
			frame:SetWidth( self.layout.Shape.size );
			
			frame.cooldown = _G[name.."Cooldown"];
			frame.icon     = _G[name.."Icon"];
			frame.count    = _G[name.."Count"];
			
			frame.duration = CreateFrame( "Frame", "$parentDuration", frame );
			frame.duration:SetAllPoints( frame );
			frame.duration:SetFrameLevel( frame.cooldown:GetFrameLevel()+1 );
			frame.remains  = frame.duration:CreateFontString( "$parentText" );
			frame.remains:SetFont( "Fonts\\ARIALN.TTF", frame:GetHeight() * 0.5, "OUTLINE" );
			frame.remains:SetTextColor( color.r, color.g, color.b, 1 );
			frame.remains:SetJustifyH( "CENTER" );
			frame.remains:SetJustifyV( "MIDDLE" );
			frame.remains:SetPoint( "CENTER", frame.duration, "CENTER", 0, 0 );
			frame.remains:SetPoint( "CENTER", frame.duration, "CENTER", 0, 0 );
			
			frame.border = CreateFrame( "Frame", "$parentBorder", frame );
			frame.border:SetAllPoints( frame );
			frame.border.texture = frame.border:CreateTexture();
			frame.border.texture:SetAllPoints( frame.border );
			frame.border.texture:SetTexture( "Interface\\AddOns\\nUI\\Units\\Art\\nUI_BuffBorder.blp" );
			frame.border.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
			frame.border.texture:SetAlpha( 1 );
		end

		buff.frame    = frame;
		buff.frame.id = buff.id;			

		-- set up the border color/texture for it
		
		local can_attack        = UnitCanAttack( "player", unit_id );
		local highlight_generic = (can_attack and not buff.harmful) or (not can_attack and buff.harmful);
		
		if self.layout.highlight ~= "no" and (highlight_generic or buff.type) then
		
			local color = colors[buff.type or "Debuff"];

			frame.border:Show();
			frame.border.texture:SetVertexColor( color.r, color.g, color.b, 1 );
			
			if self.dispellCallback then
				
				self:dispellCallback( buff.type or "Debuff", color );
				
			end
		
		else
		
			frame.border:Hide();
			
		end
		
		-- set the icon texture for this buff
		
		frame.icon:SetTexture( buff.icon );
		
		-- if it has a duration, then set a cooldown
		
		if buff.duration and buff.duration > 0 then
			
			local starts = buff.now - (buff.duration - buff.remains);
			
			if self.layout.anim ~= "no" then
				frame.cooldown:Show();
				CooldownFrame_SetTimer( frame.cooldown, starts, buff.duration, 1 );
			end
			
			if self.layout.timers ~= "no" then
					
				frame.remains.ends  = buff.now + buff.remains;
				frame.remains.timer = 0;
				
				frame.duration:SetScript( "OnUpdate",
					function( who, elapsed )
						frame.remains.timer = frame.remains.timer + elapsed;
						
						if frame.remains.timer > 0.08 then -- update at 12.5fps
						
							frame.remains.timer = 0;
							local remains = frame.remains.ends - GetTime();
							
							if remains < 10 then 
								buff.is_flashing = true;
								local color = self.layout.Colors.Flashing;
								frame.remains:SetTextColor( color.r, color.g, color.b, 1 );
							end
							
							if remains <= 0 then frame.remains:SetText( "" );
							else frame.remains:SetText( nUI_DurationText( remains ) );
							end
						end
					end
				);
			end

		else
			
			frame.remains:SetText( "" );
			frame.cooldown:Hide();
			
		end

		-- if the buff if good until cancelled, then hide the remaining time
		
		if buff.until_cancel == 1 then
			frame.remains:Hide();
		else
			frame.remains:Show();
		end
		
		-- if there is a count of applications, show it
		
		if buff.count > 1 then
			
			frame.count:SetText( buff.count );
			
		else
			
			frame.count:SetText( "" );
			
		end
	
		-- if this is a player buff, set up player events for it
		
		if buff.idx then
			
			frame:SetScript( "OnClick", 
				function()
					CancelPlayerBuff( buff.idx );
				end
			);

			frame:SetScript( "OnUpdate",
				function()
					if GameTooltip:IsOwned( frame ) then
						GameTooltip:SetPlayerBuff( buff.idx );
					end
				end
			);
			
			frame:SetScript( "OnEnter",
				function()
					GameTooltip:SetOwner( frame, anchor, 0, 0 );
					GameTooltip:SetPlayerBuff( buff.idx );
				end
			);					
			
			frame:RegisterForClicks( "RightButtonUp" );

		-- otherwise set up non-player events
		
		else
			
			frame:SetScript( "OnUpdate",
				function()
					if GameTooltip:IsOwned( frame ) then
						if buff.harmful then
							GameTooltip:SetUnitDebuff( unit_id, buff.id );
						else
							GameTooltip:SetUnitBuff( unit_id, buff.id );
						end
					end
				end
			);
			
			frame:SetScript( "OnEnter",
				function()
					GameTooltip:SetOwner( frame, anchor, 0, 0 );
					if buff.harmful then
						GameTooltip:SetUnitDebuff( unit_id, buff.id );
					else
						GameTooltip:SetUnitBuff( unit_id, buff.id );
					end
				end
			);					
			
		end
		
		frame:SetScript( "OnLeave",
			function()
				GameTooltip:Hide();
			end
		);
	end
	
	return frame;
	
end

-------------------------------------------------------------------------------

function prototype:Show()
	
	self.buff_frame:Show();
	
end

-------------------------------------------------------------------------------

function prototype:Hide()
	
	self.buff_frame:Hide();
	
end

-------------------------------------------------------------------------------

function prototype:SetPoint( point, relative_to, relative_pt, xOfs, yOfs )
	
	self.buff_frame:SetPoint( point, relative_to, relative_pt, xOfs, yOfs );
	
end

-------------------------------------------------------------------------------
-- this allows hot swapping the type of buff being displayed. This is used
-- primarily in the HUD where we treat the display of buffs and debuffs in
-- different ways depending on whether or not the target is friendly

function prototype:setBuffType( buff_type )
	
	self.layout.type = buff_type;
	self:scanBuffs( self.unit_id );
	
end

-------------------------------------------------------------------------------
-- if a non-nil callback is specified, for any dispellable debuffs we encounter
-- the callback will be executed passing in the debuff type as a string and the
-- color set for that type. Valid debuff types are "magic", "disease", "curse"
-- and "poison"

function prototype:setDispellCallback( callback )
	
	self.dispellCallback = callback;
	
end

-------------------------------------------------------------------------------

function prototype:iconCount()
	
	return self.num_buffs + self.num_debuffs;
	
end