﻿-- one visibility needs a brief explanation... "hidden" will prevent the region from displaying
-- any icons, but will test all debuffs on the unit and report the dispellable types back to the
-- client via the callback hook. This makes buff scanning a single source to maintain for things
-- like raid units where we want to know what dispells we can do without having a separate set of
-- debuff logic for those units. Visible = "no" means no buff checking at all.
--
-- The adapt buff type adapts the output based on whether the unit being referenced can be
-- attacked by the player. If the player can attack, then the display will show the unit's
-- buffs (which the player may want to dispell). Otherwise the diplay will show the debuffs
-- In either case, the intent of "adapt" is to show the player the spells they want to get
-- rid of on this unit (if they can). The adapt- buff is the same except it shows enemy
-- debuffs and friendly buffs. In practice, "adapt" is used by the nUI_HUD while "adapt-"
-- is used by the player, pet, target and targettarget unit frames.

nUI_BuffsLayout =
{
	name       = nil,			-- unique name for this buff frame
	visible    = "yes",			-- whether or not to display this region: yes|no|hidden
	type       = "buff",		-- what type of buffs do we show: buff|debuff|casts|adapt|adapt-|all 
	highlight  = "yes",			-- whether or not to highlight icons by spell type: yes|no
	timers     = "yes",			-- whether or not to include cooldown counters: yes|no
	anim       = "yes",			-- whether or not to include cooldown animation: yes|no
	flash      = 10,			-- when to start flashing expiring icons: 0 = never
	flash_rate = 1.5,			-- how fast to flash expiring icons (in seconds)
	
	Shape =
	{
		orient = "TOPLEFT",		-- where the first icon is placed: TOP|BOTTOM + LEFT|RIGHT
		grow   = "horizontal",	-- how does the region grow: horizontal|vertical
		rows   = 4,				-- how many rows to support: results in 40/rows columns
		size   = 25,			-- the size of each icon in the region
		gap    = 2,				-- horizontal and vertical gap between adjacent icons
	},
	
	Colors = 
	{
		Remains =				-- time remaining
		{
			r = 1,
			g = 1,
			b = 1,
		},
		Flashing =				-- time remaining while flashing
		{
			r = 1,
			g = 0.83,
			b = 0,
		},
		Debuff =				-- generic debuff background color
		{
			r = 1,
			g = 0.83,
			b = 0,
		},
		Magic =					-- background color of dispellable magic
		{
			r = 0,
			g = 0,
			b = 1,
		},
		Disease =				-- background color of dispellable disease
		{
			r = 0.6,
			g = 0.2,
			b = 0,
		},
		Poison =				-- background color of dispellable poison
		{
			r = 0,
			g = 1,
			b = 0,
		},
		Curse =					-- background color of dispellable curse
		{
			r = 1,
			g = 0,
			b = 1,
		},
	},
};

nUI.Class.BuffsLayout = nUI.Class:CreatePrototype();
local prototype       = nUI.Class.BuffsLayout.prototype;

-------------------------------------------------------------------------------
-- this may look a little like a PITA, but it makes it possible to add new values
-- at any time without having to parse the existing user config and insert, etc.

function prototype:init( user_defined )

	if not user_defined then user_defined = {}; end
	if not user_defined.Shape then user_defined.Shape = {}; end
	if not user_defined.Colors then user_defined.Colors = {}; end
	if not user_defined.Colors.Remains then user_defined.Colors.Remains = {}; end
	if not user_defined.Colors.Flashing then user_defined.Colors.Flashing = {}; end
	if not user_defined.Colors.Debuff then user_defined.Colors.Debuff = {}; end
	if not user_defined.Colors.Magic then user_defined.Colors.Magic = {}; end
	if not user_defined.Colors.Disease then user_defined.Colors.Disease = {}; end
	if not user_defined.Colors.Poison then user_defined.Colors.Poison = {}; end
	if not user_defined.Colors.Curse then user_defined.Colors.Curse = {}; end

	self.name       = user_defined.name or nUI_BuffsLayout.name;
	self.visible    = strlower( user_defined.visible or nUI_BuffsLayout.visible );
	self.type       = strlower( user_defined.type or nUI_BuffsLayout.type );
	self.highlight  = strlower( user_defined.highlight or nUI_BuffsLayout.highlight );
	self.timers     = strlower( user_defined.timers or nUI_BuffsLayout.timers );
	self.anim       = strlower( user_defined.anim or nUI_BuffsLayout.anim );
	self.flash      = tonumber( user_defined.flash or nUI_BuffsLayout.flash );
	self.flash_rate = tonumber( user_defined.flash_rate or nUI_BuffsLayout.flash_rate );

	self.Shape =
	{
		orient = strupper( user_defined.Shape.orient or nUI_BuffsLayout.Shape.orient ),
		grow   = strlower( user_defined.Shape.grow or nUI_BuffsLayout.Shape.grow ),
		rows   = tonumber( user_defined.Shape.rows or nUI_BuffsLayout.Shape.rows ),
		size   = tonumber( user_defined.Shape.size or nUI_BuffsLayout.Shape.size ),
		gap    = tonumber( user_defined.Shape.gap or nUI_BuffsLayout.Shape.gap ),
	};
	
	self.Colors =
	{
		Remains =
		{
			r = tonumber( user_defined.Colors.Remains.r or nUI_BuffsLayout.Colors.Remains.r ),
			g = tonumber( user_defined.Colors.Remains.g or nUI_BuffsLayout.Colors.Remains.g ),
			b = tonumber( user_defined.Colors.Remains.b or nUI_BuffsLayout.Colors.Remains.b ),
		},
		Flashing =
		{
			r = tonumber( user_defined.Colors.Flashing.r or nUI_BuffsLayout.Colors.Flashing.r ),
			g = tonumber( user_defined.Colors.Flashing.g or nUI_BuffsLayout.Colors.Flashing.g ),
			b = tonumber( user_defined.Colors.Flashing.b or nUI_BuffsLayout.Colors.Flashing.b ),
		},
		Debuff =
		{
			r = tonumber( user_defined.Colors.Debuff.r or nUI_BuffsLayout.Colors.Debuff.r ),
			g = tonumber( user_defined.Colors.Debuff.g or nUI_BuffsLayout.Colors.Debuff.g ),
			b = tonumber( user_defined.Colors.Debuff.b or nUI_BuffsLayout.Colors.Debuff.b ),
		},
		Magic =
		{
			r = tonumber( user_defined.Colors.Magic.r or nUI_BuffsLayout.Colors.Magic.r ),
			g = tonumber( user_defined.Colors.Magic.g or nUI_BuffsLayout.Colors.Magic.g ),
			b = tonumber( user_defined.Colors.Magic.b or nUI_BuffsLayout.Colors.Magic.b ),
		},
		Disease =
		{
			r = tonumber( user_defined.Colors.Disease.r or nUI_BuffsLayout.Colors.Disease.r ),
			g = tonumber( user_defined.Colors.Disease.g or nUI_BuffsLayout.Colors.Disease.g ),
			b = tonumber( user_defined.Colors.Disease.b or nUI_BuffsLayout.Colors.Disease.b ),
		},
		Poison =
		{
			r = tonumber( user_defined.Colors.Poison.r or nUI_BuffsLayout.Colors.Poison.r ),
			g = tonumber( user_defined.Colors.Poison.g or nUI_BuffsLayout.Colors.Poison.g ),
			b = tonumber( user_defined.Colors.Poison.b or nUI_BuffsLayout.Colors.Poison.b ),
		},
		Curse =
		{
			r = tonumber( user_defined.Colors.Curse.r or nUI_BuffsLayout.Colors.Curse.r ),
			g = tonumber( user_defined.Colors.Curse.g or nUI_BuffsLayout.Colors.Curse.g ),
			b = tonumber( user_defined.Colors.Curse.b or nUI_BuffsLayout.Colors.Curse.b ),
		},
	};
	
end