﻿-------------------------------------------------------------------------------
-- local cache

local RequestBattlefieldPositions = RequestBattlefieldPositions;
local UnitName                    = UnitName;
local GetRealmName                = GetRealmName;
local hooksecurefunc              = hooksecurefunc;
local CreateFrame                 = CreateFrame;
local MouseIsOver                 = MouseIsOver;
local GetPlayerMapPosition        = GetPlayerMapPosition;
local SetMapToCurrentZone         = SetMapToCurrentZone;
local PlaySound                   = PlaySound;
local CloseDropDownMenus          = CloseDropDownMenus;
local GetGameTime                 = GetGameTime;
local ReloadUI                    = ReloadUI;
local GetBinding                  = GetBinding;
local GetNumBindings              = GetNumBindings;
local LoadAddOn                   = LoadAddOn;
local InCombatLockdown            = InCombatLockdown;
local GetScreenHeight             = GetScreenHeight;
local GetScreenWidth              = GetScreenWidth;

-------------------------------------------------------------------------------

nUI = 
{
	version      = "0.0.7 (alpha)",
    copyright    = "Copyright (C) 2008, All Rights Reserved",
	first_load   = true,
	vars_loaded  = false,
	scale        = 1,
	is_hidden    = false,
    player_name  = nil,
	view_height  = 0,
	mmStateCache = nil,
	last_binding = 1,
	chat_zoomed  = false,
	info_zoomed  = false,
	suppress     = true,
	run_time     = 0,
	buttonbag    = 0,
	
	Timers =
	{
		player_coords = 0.5,
		clock         = 1.0,
		buttonbag     = 0,
	},
	
	UnitLayouts = {},
	BarLayouts  = {},
	BarLayouts2 = {},
	Units       = {},
	Bars        = {},
	Buttons     = {},
	ButtonBag   = {},
	ExtraBars   = {},
	SpellStatus = {},
	Cooldowns   = {},
	AddOn       = {},
}

-------------------------------------------------------------------------------

nUI_UnitLayouts = {};
nUI_BarLayouts  = {};

-------------------------------------------------------------------------------
-- this function massages the user configuration to ensure all the pieces we
-- have to have are present, the string case is what we expect, numbers are
-- numbers, etc. It also allows us to easily add new options to existing
-- configurations without having to parse them, etc.

function nUI_Config_Init()

	if not nUI_Config then nUI_Config = {}; end
	if not nUI_Config.Movers then nUI_Config.Movers = {}; end
	if not nUI_Config.ButtonBag then nUI_Config.ButtonBag = {}; end
	if not nUI_Config.ButtonBag.Excludes then nUI_Config.ButtonBag.Excludes = {}; end
	if not nUI_Config.ButtonBag.Includes then nUI_Config.ButtonBag.Includes = {}; end
	if not nUI_Config.ButtonBag.Buttons then nUI_Config.ButtonBag.Buttons = {}; end
	if not nUI_Config.ButtonBag.Logging then nUI_Config.ButtonBag.Logging = {}; end
	if not nUI_Config.Colors then nUI_Config.Colors = {}; end
	if not nUI_Config.Colors.OOR then nUI_Config.Colors.OOR = {}; end
	if not nUI_Config.Colors.OOM then nUI_Config.Colors.OOM = {}; end
	if not nUI_Config.Colors.CDC1 then nUI_Config.Colors.CDC1 = {}; end
	if not nUI_Config.Colors.CDC2 then nUI_Config.Colors.CDC2 = {}; end
	
	nUI_Config.hidden         = strlower( nUI_Config.hidden or "no" );
	nUI_Config.debug          = tonumber( nUI_Config.debug or 0 );
	nUI_Config.bag_scale      = tonumber( nUI_Config.bag_scale or 0.9 );
	nUI_Config.chatfont_size  = tonumber( nUI_Config.chatfont_size or 12 );
	nUI_Config.unit_anim      = strlower( nUI_Config.unit_anim or "anim" );
	nUI_Config.party_pets     = strlower( nUI_Config.party_pets or "yes" );
	nUI_Config.party_targets  = strlower( nUI_Config.party_targets or "yes" );
	nUI_Config.party_ptargets = strlower( nUI_Config.party_ptargets or "yes" );
	nUI_Config.auto_ding      = strlower( nUI_Config.auto_ding or "yes" );
	nUI_Config.yOfs           = tonumber( nUI_Config.yOfs or 0 );
	nUI_Config.scale          = nUI_Config.scale or nil;

	nUI_Config.Colors.OOR.r   = tonumber( nUI_Config.Colors.OOR.r or 1 );
	nUI_Config.Colors.OOR.g   = tonumber( nUI_Config.Colors.OOR.g or 0 );
	nUI_Config.Colors.OOR.b   = tonumber( nUI_Config.Colors.OOR.b or 0 );
	nUI_Config.Colors.OOM.r   = tonumber( nUI_Config.Colors.OOM.r or 0 );
	nUI_Config.Colors.OOM.g   = tonumber( nUI_Config.Colors.OOM.g or 0 );
	nUI_Config.Colors.OOM.b   = tonumber( nUI_Config.Colors.OOM.b or 1 );
	nUI_Config.Colors.CDC1.r  = tonumber( nUI_Config.Colors.CDC1.r or 1 );
	nUI_Config.Colors.CDC1.g  = tonumber( nUI_Config.Colors.CDC1.g or 1 );
	nUI_Config.Colors.CDC1.b  = tonumber( nUI_Config.Colors.CDC1.b or 0 );
	nUI_Config.Colors.CDC2.r  = tonumber( nUI_Config.Colors.CDC2.r or 1 );
	nUI_Config.Colors.CDC2.g  = tonumber( nUI_Config.Colors.CDC2.g or 0.6 );
	nUI_Config.Colors.CDC2.b  = tonumber( nUI_Config.Colors.CDC2.b or 0.6 );
end

-------------------------------------------------------------------------------

nUI_Config = nUI_Config_Init();

-------------------------------------------------------------------------------
--                         Construct our class manager                       --
-------------------------------------------------------------------------------

nUI.Class = {}

local function objectConstructor( class, ... )
	
	local object = {};
	setmetatable( object, class.mt );
	
	if object.init then
		object.init( object, ... );
	end
	
	return object
	
end

-------------------------------------------------------------------------------

function nUI.Class:CreatePrototype( inherit )
    
	local class, prototype = {}, {};
	
	if inherit then
		setmetatable( prototype, inherit.mt );
		prototype.super = inherit;
	end
	
	class.prototype = prototype;
	class.new       = objectConstructor;
	class.mt        = { __index = prototype };
	
	return class;
	
end

-------------------------------------------------------------------------------
--             Base system management and support routines                   --
-------------------------------------------------------------------------------

function nUI:debugMsg( level, msg, r, g, b )

	if not msg then
		msg   = level;
		level = 0;
	end
	
	if level <= nUI_Config.debug
	then
		local cR = r or 1;
		local cG = g or 0.25;
		local cB = b or 0.25;
	
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..msg, cR, cG, cB );
	end
	
end

-------------------------------------------------------------------------------
-- who woulda thunk there was no round function... yeah, I know... it's a hack

function nUI:round( value, decimals )

	local multiplier = 1;
	
	for i = 1, decimals, 1 do
		multiplier = multiplier * 10;
	end
	
	return math.floor( value * multiplier + 0.5 ) / multiplier;
end


-------------------------------------------------------------------------------

function nUI:setScale()

	local screen_res    =({GetScreenResolutions()})[GetCurrentResolution()];
	local width, height = strmatch( screen_res, "(%d+)x(%d+)" );

	nUI.disp_width  = math.floor( width );
	nUI.disp_height = math.floor( height );
	nUI.scalar      = nUI.disp_width / 2560;

	if nUI_Config.scale then
		nUI.scale = nUI_Config.scale;
	elseif nUI_Scale[nUI.disp_width] and nUI_Scale[nUI.disp_width][nUI.disp_height] then
		nUI.scale = nUI_Scale[nUI.disp_width][nUI.disp_height];
	else
		nUI.scale = 0.65;
		nUI:debugMsg( 0, "no defined scale for ("..nUI.disp_width.."x"..nUI.disp_height..") resolution\nTry using '/nui scale' {n} to adjust your scale\nValues of {n} are usually between 0.62 and 1.2\nAdjust {n} until the dashboard fits your screen\nLet me know your resolution and scale when you get it\nso I can add it to my table. kscottpiel@gmail.com" );
	end

	nUI.scale = nUI.scalar * nUI.scale;
	nUI_MasterFrame:SetScale( nUI.scale );
	nUI_Console:SetScale( 0.8 );
end

-------------------------------------------------------------------------------
--                               Event management                            --
-------------------------------------------------------------------------------

function nUI:onLoad()

	nUI.playerName = UnitName( "player" );
	nUI.realmName  = GetRealmName():trim();

	DEFAULT_CHAT_FRAME:AddMessage( "Welcome back to "..nUI.realmName.." "..nUI.playerName.."...\nnUI version "..nUI.version.." is loaded!\nType \"/nui\" for a list of available nUI commands.", 0, 1.0, 0.50 );

	SlashCmdList["NUI"] = function( msg ) self:slashCommand( msg );	end
	SLASH_NUI1 = "/nui";

	hooksecurefunc( "updateContainerFrameAnchors", nUI_manageBags );
	hooksecurefunc( "ActionButton_OnUpdate",       nUI_Button_OnUpdate );
	hooksecurefunc( "ActionButton_UpdateUsable",   nUI_Button_UpdateUsable );
	hooksecurefunc( "ActionButton_Update",         nUI_Button_Update );
	hooksecurefunc( "CooldownFrame_SetTimer",      nUI_Button_CooldownTimer );
	
	nUI:registerEvents();
	
end

-------------------------------------------------------------------------------

function nUI:registerEvents()

	nUI_MasterFrame:RegisterEvent( "VARIABLES_LOADED" );
	nUI_MasterFrame:RegisterEvent( "PLAYER_ENTERING_WORLD" );
	nUI_MasterFrame:RegisterEvent( "PLAYER_LEAVING_WORLD" );
	
end

-------------------------------------------------------------------------------

function nUI:onEvent()
	
	if     event == "VARIABLES_LOADED"      then nUI:variablesLoaded();
	elseif event == "PLAYER_ENTERING_WORLD" then nUI:enteringWorld();
	elseif event == "PLAYER_LEAVING_WORLD"  then nUI:leavingWorld();
	elseif event == "PLAYER_TARGET_CHANGED" then nUI:updateTarget();
	end
	
end

-------------------------------------------------------------------------------

function nUI:updateTarget()
	
	nUI.Units["target"]:checkGUID();
	nUI.Units["targettarget"]:checkGUID();
	nUI.Units["targettargettarget"]:checkGUID();

end

-------------------------------------------------------------------------------

function nUI:variablesLoaded()
	
	local	portrait_mode = "anim";
	local	id;

	if not nUI.vars_loaded then
	
		nUI.vars_loaded = true;
		
		-- make sure we have everything we expect and need in the config

		nUI_Config_Init();
		
		-- initialize player unit frame layout
		
		id = "player"; 
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                         = {};
			nUI.UnitLayouts[id].Portrait                = {};
			nUI.UnitLayouts[id].StatusBars              = {};
			nUI.UnitLayouts[id].Origin                  = {};
			nUI.UnitLayouts[id].BuffInfo                = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs          = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs.location = "detatched";
			nUI.UnitLayouts[id].BuffInfo.Buffs.anchor   = "BOTTOMRIGHT";
			nUI.UnitLayouts[id].BuffInfo.Buffs.flash    = 10;
			nUI.UnitLayouts[id].BuffInfo.Buffs.timers   = "yes";
			nUI.UnitLayouts[id].BuffInfo.Buffs.xOfs     = -425;
			nUI.UnitLayouts[id].BuffInfo.Buffs.yOfs     = 150;
			nUI.UnitLayouts[id].BuffInfo.Buffs.width    = 450;
			nUI.UnitLayouts[id].orientation             = "left";
			nUI.UnitLayouts[id].show_border             = "yes";
			nUI.UnitLayouts[id].Portrait.visible        = portrait_mode;
			nUI.UnitLayouts[id].StatusBars.visible      = "no";
			nUI.UnitLayouts[id].Origin.anchor           = "BOTTOMRIGHT";
			nUI.UnitLayouts[id].Origin.xOfs             = -168;
			nUI.UnitLayouts[id].Origin.yOfs             = 155;
			nUI.UnitLayouts[id]                         = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
		end
		
		-- initialize pet unit frame layout
	
		id = "pet";
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                         = {};
			nUI.UnitLayouts[id].Portrait                = {};
			nUI.UnitLayouts[id].StatusBars              = {};
			nUI.UnitLayouts[id].Size                    = {};
			nUI.UnitLayouts[id].Origin                  = {};
			nUI.UnitLayouts[id].BuffInfo                = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs          = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs.location = "detatched";
			nUI.UnitLayouts[id].BuffInfo.Buffs.anchor   = "BOTTOMRIGHT";
			nUI.UnitLayouts[id].BuffInfo.Buffs.flash    = 10;
			nUI.UnitLayouts[id].BuffInfo.Buffs.xOfs     = -910;
			nUI.UnitLayouts[id].BuffInfo.Buffs.yOfs     = 150;
			nUI.UnitLayouts[id].BuffInfo.Buffs.width    = 300;
			nUI.UnitLayouts[id].orientation             = "left";
			nUI.UnitLayouts[id].show_reaction           = "no";
			nUI.UnitLayouts[id].Portrait.visible        = portrait_mode;
			nUI.UnitLayouts[id].StatusBars.visible      = "no";
			nUI.UnitLayouts[id].Size.width              = "330";
			nUI.UnitLayouts[id].Size.height             = "97";
			nUI.UnitLayouts[id].Origin.anchor           = "BOTTOMRIGHT";
			nUI.UnitLayouts[id].Origin.xOfs             = -168;
			nUI.UnitLayouts[id].Origin.yOfs             = 155;
			nUI.UnitLayouts[id]                         = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
		end
		
		-- initialize pet target frame layout
		
		id = "pettarget";
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                    = {};
			nUI.UnitLayouts[id].Portrait           = {};
			nUI.UnitLayouts[id].StatusBars         = {};
			nUI.UnitLayouts[id].Size               = {};
			nUI.UnitLayouts[id].Origin             = {};
			nUI.UnitLayouts[id].orientation        = "right";
			nUI.UnitLayouts[id].show_detail        = "yes";
			nUI.UnitLayouts[id].Portrait.visible   = "no";
			nUI.UnitLayouts[id].StatusBars.visible = "yes";
			nUI.UnitLayouts[id].StatusBars.spell   = "none";
			nUI.UnitLayouts[id].StatusBars.percent = "none";
			nUI.UnitLayouts[id].StatusBars.values  = "none";
			nUI.UnitLayouts[id].StatusBars.total   = "none";
			nUI.UnitLayouts[id].Size.width         = "212";
			nUI.UnitLayouts[id].Size.height        = "62";
			nUI.UnitLayouts[id].Size.two_line      = "yes";
			nUI.UnitLayouts[id].Origin.anchor      = "BOTTOMRIGHT";
			nUI.UnitLayouts[id].Origin.xOfs        = -168;
			nUI.UnitLayouts[id].Origin.yOfs        = 155;
			nUI.UnitLayouts[id]                    = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
			
		end
		
		-- initialize target unit frame layout
		
		id = "target";
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                         = {};
			nUI.UnitLayouts[id].Portrait                = {};
			nUI.UnitLayouts[id].StatusBars              = {};
			nUI.UnitLayouts[id].Size                    = {};
			nUI.UnitLayouts[id].Origin                  = {};
			nUI.UnitLayouts[id].BuffInfo                = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs          = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs.location = "detatched";
			nUI.UnitLayouts[id].BuffInfo.Buffs.type     = "adapt-";
			nUI.UnitLayouts[id].BuffInfo.Buffs.anchor   = "BOTTOMLEFT";
			nUI.UnitLayouts[id].BuffInfo.Buffs.flash    = 10;
			nUI.UnitLayouts[id].BuffInfo.Buffs.timers   = "yes";
			nUI.UnitLayouts[id].BuffInfo.Buffs.xOfs     = 425;
			nUI.UnitLayouts[id].BuffInfo.Buffs.yOfs     = 150;
			nUI.UnitLayouts[id].BuffInfo.Buffs.width    = 450;
			nUI.UnitLayouts[id].orientation             = "right";
			nUI.UnitLayouts[id].show_border             = "yes";
			nUI.UnitLayouts[id].Portrait.visible        = portrait_mode;
			nUI.UnitLayouts[id].StatusBars.visible      = "no";
			nUI.UnitLayouts[id].elite_icon              = "yes";
			nUI.UnitLayouts[id].Origin.anchor           = "BOTTOMLEFT";
			nUI.UnitLayouts[id].Origin.xOfs             = 168;
			nUI.UnitLayouts[id].Origin.yOfs             = 155;
			nUI.UnitLayouts[id]                         = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
		end
		
		-- initialize target of target frame layout
		
		id = "targettarget";
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                         = {};
			nUI.UnitLayouts[id].Portrait                = {};
			nUI.UnitLayouts[id].StatusBars              = {};
			nUI.UnitLayouts[id].Size                    = {};
			nUI.UnitLayouts[id].Origin                  = {};
			nUI.UnitLayouts[id].BuffInfo                = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs          = {};
			nUI.UnitLayouts[id].BuffInfo.Buffs.location = "detatched";
			nUI.UnitLayouts[id].BuffInfo.Buffs.anchor   = "BOTTOMLEFT";
			nUI.UnitLayouts[id].BuffInfo.Buffs.flash    = 10;
			nUI.UnitLayouts[id].BuffInfo.Buffs.xOfs     = 910;
			nUI.UnitLayouts[id].BuffInfo.Buffs.yOfs     = 150;
			nUI.UnitLayouts[id].BuffInfo.Buffs.width    = 300;
			nUI.UnitLayouts[id].orientation             = "right";
			nUI.UnitLayouts[id].Portrait.visible        = portrait_mode;
			nUI.UnitLayouts[id].StatusBars.visible      = "no";
			nUI.UnitLayouts[id].Size.width              = "330";
			nUI.UnitLayouts[id].Size.height             = "97";
			nUI.UnitLayouts[id].Origin.anchor           = "BOTTOMLEFT";
			nUI.UnitLayouts[id].Origin.xOfs             = 168;
			nUI.UnitLayouts[id].Origin.yOfs             = 155;
			nUI.UnitLayouts[id]                         = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
		end
	
		-- initialize target of target of target frame layout
		
		id = "targettargettarget";
		
		if nUI_UnitLayouts[id] ~= nil then
			nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
		else
			nUI.UnitLayouts[id]                    = {};
			nUI.UnitLayouts[id].Portrait           = {};
			nUI.UnitLayouts[id].StatusBars         = {};
			nUI.UnitLayouts[id].Size               = {};
			nUI.UnitLayouts[id].Origin             = {};
			nUI.UnitLayouts[id].orientation        = "left";
			nUI.UnitLayouts[id].show_detail        = "yes";
			nUI.UnitLayouts[id].Portrait.visible   = "no";
			nUI.UnitLayouts[id].StatusBars.visible = "yes";
			nUI.UnitLayouts[id].StatusBars.spell   = "none";
			nUI.UnitLayouts[id].StatusBars.percent = "none";
			nUI.UnitLayouts[id].StatusBars.values  = "none";
			nUI.UnitLayouts[id].StatusBars.total   = "none";
			nUI.UnitLayouts[id].Size.width         = "212";
			nUI.UnitLayouts[id].Size.height        = "62";
			nUI.UnitLayouts[id].Size.two_line      = "yes";
			nUI.UnitLayouts[id].Origin.anchor      = "BOTTOMLEFT";
			nUI.UnitLayouts[id].Origin.xOfs        = 171;
			nUI.UnitLayouts[id].Origin.yOfs        = 157;
			nUI.UnitLayouts[id]                    = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
		end	

		-- party 1 - 4
		
		local pet_targets = nUI_Config.party_ptargets;
		
		if nUI_Config.party_pets == "no" then
			pet_targets = "no";
		end
		
		for i=1, 4, 1 do
	
			-- the party member
			
			id = "party"..i;
			
			if nUI_UnitLayouts[id] ~= nil then
				nUI.UnitLayouts[id] = nUI.Class.UnitLayout:new( id, nUI_UnitLayouts[id] );
			else
				nUI.UnitLayouts[id]                           = {};
				nUI.UnitLayouts[id].Portrait                  = {};
				nUI.UnitLayouts[id].StatusBars                = {};
				nUI.UnitLayouts[id].CastingBars               = {};
				nUI.UnitLayouts[id].Size                      = {};
				nUI.UnitLayouts[id].Origin                    = {};
				nUI.UnitLayouts[id].BuffInfo                  = {};
				nUI.UnitLayouts[id].BuffInfo                  = {};
				nUI.UnitLayouts[id].BuffInfo.Buffs            = {};
				nUI.UnitLayouts[id].BuffInfo.Debuffs          = {};
				nUI.UnitLayouts[id].BuffInfo.Buffs.rows       = 4;
				nUI.UnitLayouts[id].BuffInfo.Buffs.location   = "bottom";
				nUI.UnitLayouts[id].BuffInfo.Debuffs.rows     = 4;
				nUI.UnitLayouts[id].BuffInfo.Debuffs.location = "top";
				nUI.UnitLayouts[id].id                        = i;
				nUI.UnitLayouts[id].orientation               = "left";
				nUI.UnitLayouts[id].show_reaction             = "no";
				nUI.UnitLayouts[id].Portrait.visible          = portrait_mode;
				nUI.UnitLayouts[id].StatusBars.visible        = "yes";
				nUI.UnitLayouts[id].CastingBars.visible       = "yes";
				nUI.UnitLayouts[id].Size.width                = "340";
				nUI.UnitLayouts[id].Size.height               = "120";
				nUI.UnitLayouts[id].Origin.anchor             = "BOTTOMRIGHT";
				nUI.UnitLayouts[id].Origin.xOfs               = -930;
				nUI.UnitLayouts[id].Origin.yOfs               = 1425 - (i-1) * 250;
				nUI.UnitLayouts[id]                           = nUI.Class.UnitLayout:new( id, nUI.UnitLayouts[id] );
			end
	
			-- the party member's target
			
			local tid = id.."target";
			
			if nUI_UnitLayouts[tid] ~= nil then
				nUI.UnitLayouts[tid] = nUI.Class.UnitLayout:new( tid, nUI_UnitLayouts[tid] );
			else
				nUI.UnitLayouts[tid]                    = {};
				nUI.UnitLayouts[tid].Portrait           = {};
				nUI.UnitLayouts[tid].StatusBars         = {};
				nUI.UnitLayouts[tid].Size               = {};
				nUI.UnitLayouts[tid].Origin             = {};
				nUI.UnitLayouts[tid].orientation        = "left";
				nUI.UnitLayouts[tid].show_detail        = "no";
				nUI.UnitLayouts[tid].show_reaction      = "no";
				nUI.UnitLayouts[tid].Portrait.visible   = "no";
				nUI.UnitLayouts[tid].StatusBars.visible = "no";
				nUI.UnitLayouts[tid].Size.width         = "175";
				nUI.UnitLayouts[tid].Size.height        = nUI.UnitLayouts[id].Size.height / 3.5;
				nUI.UnitLayouts[tid].Origin.anchor      = "BOTTOMLEFT";
				nUI.UnitLayouts[tid].Origin.xOfs        = nUI.UnitLayouts[id].Origin.xOfs - 6;
				nUI.UnitLayouts[tid].Origin.yOfs        = nUI.UnitLayouts[id].Origin.yOfs + 2;
				nUI.UnitLayouts[tid]                    = nUI.Class.UnitLayout:new( tid, nUI.UnitLayouts[tid] );
			end
	
			-- the party member's pet
			
			local pid = "partypet"..i;
			
			if nUI_UnitLayouts[pid] ~= nil then
				nUI.UnitLayouts[pid] = nUI.Class.UnitLayout:new( pid, nUI_UnitLayouts[pid] );
			else
				nUI.UnitLayouts[pid]                    = {};
				nUI.UnitLayouts[pid].Portrait           = {};
				nUI.UnitLayouts[pid].StatusBars         = {};
				nUI.UnitLayouts[pid].Size               = {};
				nUI.UnitLayouts[pid].Origin             = {};
				nUI.UnitLayouts[pid].id                 = i;
				nUI.UnitLayouts[pid].orientation        = "left";
				nUI.UnitLayouts[pid].is_visible         = nUI_Config.party_pets;
				nUI.UnitLayouts[pid].show_detail        = "yes";
				nUI.UnitLayouts[pid].show_reaction      = "no";
				nUI.UnitLayouts[pid].show_class         = "no";
				nUI.UnitLayouts[pid].Portrait.visible   = "no";
				nUI.UnitLayouts[pid].StatusBars.visible = "yes";
				nUI.UnitLayouts[pid].StatusBars.percent = "center";
				nUI.UnitLayouts[pid].StatusBars.values  = "no";
				nUI.UnitLayouts[pid].StatusBars.total   = "no";
				nUI.UnitLayouts[pid].Size.width         = nUI.UnitLayouts[tid].Size.width;
				nUI.UnitLayouts[pid].Size.height        = nUI.UnitLayouts[id].Size.height - nUI.UnitLayouts[tid].Size.height + 4;
				nUI.UnitLayouts[pid].Size.two_line      = "yes";
				nUI.UnitLayouts[pid].Origin.anchor      = "BOTTOMLEFT";
				nUI.UnitLayouts[pid].Origin.xOfs        = nUI.UnitLayouts[tid].Origin.xOfs;
				nUI.UnitLayouts[pid].Origin.yOfs        = nUI.UnitLayouts[tid].Origin.yOfs + nUI.UnitLayouts[tid].Size.height - 6;
				nUI.UnitLayouts[pid]                    = nUI.Class.UnitLayout:new( pid, nUI.UnitLayouts[pid] );
			end
				
			-- the party member's pet's target
			
			local ptid = "partypet"..i.."target";
			
			if nUI_UnitLayouts[ptid] ~= nil then
				nUI.UnitLayouts[ptid] = nUI.Class.UnitLayout:new( ptid, nUI_UnitLayouts[ptid] );
			else
				nUI.UnitLayouts[ptid]                    = {};
				nUI.UnitLayouts[ptid].Portrait           = {};
				nUI.UnitLayouts[ptid].StatusBars         = {};
				nUI.UnitLayouts[ptid].Size               = {};
				nUI.UnitLayouts[ptid].Origin             = {};
				nUI.UnitLayouts[ptid].orientation        = "left";
				nUI.UnitLayouts[ptid].show_detail        = "no";
				nUI.UnitLayouts[ptid].is_visible         = pet_targets;
				nUI.UnitLayouts[ptid].Portrait.visible   = "no";
				nUI.UnitLayouts[ptid].StatusBars.visible = "no";
				nUI.UnitLayouts[ptid].Size.width         = nUI.UnitLayouts[pid].Size.width;
				nUI.UnitLayouts[ptid].Size.height        = nUI.UnitLayouts[pid].Size.height / 3;
				nUI.UnitLayouts[ptid].Origin.anchor      = "BOTTOMLEFT";
				nUI.UnitLayouts[ptid].Origin.xOfs        = nUI.UnitLayouts[tid].Origin.xOfs;
				nUI.UnitLayouts[ptid].Origin.yOfs        = nUI.UnitLayouts[pid].Origin.yOfs + nUI.UnitLayouts[pid].Size.height - 6;
				nUI.UnitLayouts[ptid]                    = nUI.Class.UnitLayout:new( ptid, nUI.UnitLayouts[ptid] );
			end
			
		end	
	
		-- the special bar (pet bar)
	
		nUI_BarLayouts = {}; -- emergency fix
		
		id = "nUI_PetActionBarFrame";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else 
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "Secondary Action";
			nUI.BarLayouts[id].button_src    = "PetActionButton";
			nUI.BarLayouts[id].btn_type      = "BONUSACTIONBUTTON";
			nUI.BarLayouts[id].inherits      = "PetActionButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOM";
			nUI.BarLayouts[id].Origin.yOfs   = 187;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 10;
			nUI.BarLayouts[id].btn_size      = 39;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end
		
		nUI.BarLayouts[id].parent_bar = _G["PetActionBarFrame"];		

		-- the stance bar (used by warriors, pallys,!@#$ droods, etc)
	
		id = "nUI_ShapeshiftBarFrame";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "Special Action";
			nUI.BarLayouts[id].button_src    = "ShapeshiftButton";
			nUI.BarLayouts[id].btn_type      = "SHAPESHIFTBUTTON";
			nUI.BarLayouts[id].inherits      = "ShapeshiftButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOM";
			nUI.BarLayouts[id].Origin.yOfs   = 187;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 10;
			nUI.BarLayouts[id].btn_size      = 39;
			nUI.BarLayouts[id].visible       = "no";
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end
		
		nUI.BarLayouts[id].parent_bar = _G["ShapeshiftBarFrame"];
		
		-- the stance bar (used by warriors, pallys,!@#$ droods, etc)
	
		id = "nUI_PossessBarFrame";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "Possess Bar";
			nUI.BarLayouts[id].button_src    = "PossessButton";
			nUI.BarLayouts[id].inherits      = "PossessButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOM";
			nUI.BarLayouts[id].Origin.yOfs   = 187;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 10;
			nUI.BarLayouts[id].btn_size      = 39;
			nUI.BarLayouts[id].visible       = "no";
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end
		
		nUI.BarLayouts[id].parent_bar = _G["PossessBarFrame"];
		
		-- the default action bar
	
		id = "nUI_MainMenuBarArtFrame";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg1 / Action";
			nUI.BarLayouts[id].button_src    = "ActionButton";
			nUI.BarLayouts[id].btn_type      = "ACTIONBUTTON";
			nUI.BarLayouts[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOM";
			nUI.BarLayouts[id].Origin.yOfs   = 122;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 12;
			nUI.BarLayouts[id].btn_size      = 55;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end

		nUI.BarLayouts[id].parent_bar = _G["MainMenuBarArtFrame"];
		
		-- the default action bar
	
		id = "nUI_BonusActionBarFrame";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg1 / Action";
			nUI.BarLayouts[id].button_src    = "BonusActionButton";
			nUI.BarLayouts[id].btn_type      = "ACTIONBUTTON";
			nUI.BarLayouts[id].inherits      = "BonusActionButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOM";
			nUI.BarLayouts[id].Origin.yOfs   = 122;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 12;
			nUI.BarLayouts[id].btn_size      = 55;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end

		nUI.BarLayouts[id].parent_bar = _G["BonusActionBarFrame"];
		
		-- the Bliz "bottom left" bar
	
		id = "nUI_MultiBarBottomLeft";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg6 / BottomLeft";
			nUI.BarLayouts[id].button_src    = "MultiBarBottomLeftButton";
			nUI.BarLayouts[id].btn_type      = "MULTIACTIONBAR1BUTTON";
			nUI.BarLayouts[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOMRIGHT";
			nUI.BarLayouts[id].Origin.yOfs   = 55;
			nUI.BarLayouts[id].Origin.xOfs   = -168;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 12;
			nUI.BarLayouts[id].btn_size      = 54;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end
	
		nUI.BarLayouts[id].parent_bar = _G["MultiBarBottomLeft"];
		
		-- the Bliz "bottom right" bar
	
		id = "nUI_MultiBarBottomRight";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg5 / BottomRight";
			nUI.BarLayouts[id].button_src    = "MultiBarBottomRightButton";
			nUI.BarLayouts[id].btn_type      = "MULTIACTIONBAR2BUTTON";
			nUI.BarLayouts[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOMLEFT";
			nUI.BarLayouts[id].Origin.yOfs   = 55;
			nUI.BarLayouts[id].Origin.xOfs   = 168;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 12;
			nUI.BarLayouts[id].btn_size      = 54;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end

		nUI.BarLayouts[id].parent_bar = _G["MultiBarBottomRight"];
		
		-- the Bliz "right 1" bar (aka "left" bar in the Bliz code)
	
		id = "nUI_MultiBarLeft";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg3 / Right 2";
			nUI.BarLayouts[id].button_src    = "MultiBarRightButton";
			nUI.BarLayouts[id].btn_type      = "MULTIACTIONBAR3BUTTON";
			nUI.BarLayouts[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "TOPRIGHT";
			nUI.BarLayouts[id].Origin.yOfs   = -110;
			nUI.BarLayouts[id].Origin.xOfs   = -168;
			nUI.BarLayouts[id].rows          = 2;
			nUI.BarLayouts[id].cols          = 6;
			nUI.BarLayouts[id].btn_size      = 42.8;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end

		nUI.BarLayouts[id].parent_bar = _G["MultiBarLeft"];
		
		-- the Bliz "right 2" bar (aka "right" bar in the Bliz code)
	
		id = "nUI_MultiBarRight";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "pg4 / Right";
			nUI.BarLayouts[id].button_src    = "MultiBarLeftButton";
			nUI.BarLayouts[id].btn_type      = "MULTIACTIONBAR4BUTTON";
			nUI.BarLayouts[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "TOPLEFT";
			nUI.BarLayouts[id].Origin.yOfs   = -110;
			nUI.BarLayouts[id].Origin.xOfs   = 168;
			nUI.BarLayouts[id].rows          = 2;
			nUI.BarLayouts[id].cols          = 6;
			nUI.BarLayouts[id].btn_size      = 42.8;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );
		end

		nUI.BarLayouts[id].parent_bar = _G["MultiBarRight"];
	
		-- bags
	
		id = "nUI_Bar_Bags";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts[id]               = {};
			nUI.BarLayouts[id].Origin        = {};
			nUI.BarLayouts[id].name          = id;
			nUI.BarLayouts[id].label         = "Bags";
			nUI.BarLayouts[id].btn_type      = "TOGGLEBAG";
			nUI.BarLayouts[id].inherits      = "BagSlotButtonTemplate";
			nUI.BarLayouts[id].Origin.anchor = "BOTTOMRIGHT";
			nUI.BarLayouts[id].Origin.yOfs   = 55;
			nUI.BarLayouts[id].Origin.xOfs   = 1269;
			nUI.BarLayouts[id].rows          = 1;
			nUI.BarLayouts[id].cols          = 5;
			nUI.BarLayouts[id].key_ring      = "left";
			nUI.BarLayouts[id].btn_size      = 54;
			nUI.BarLayouts[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts[id] );			
		end
		
		-- bonus bar 1
	
		id = "nUI_Bar1";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts2[id]               = {};
			nUI.BarLayouts2[id].Origin        = {};
			nUI.BarLayouts2[id].name          = id;
			nUI.BarLayouts2[id].label         = "nUI Bar 1";
			nUI.BarLayouts2[id].button_src    = "nUIBar1Button";
			nUI.BarLayouts2[id].btn_type      = "NUIBAR1BUTTON";
			nUI.BarLayouts2[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts2[id].Origin.anchor = "TOPLEFT";
			nUI.BarLayouts2[id].Origin.yOfs   = -110;
			nUI.BarLayouts2[id].Origin.xOfs   = -682;
			nUI.BarLayouts2[id].rows          = 2;
			nUI.BarLayouts2[id].cols          = 5;
			nUI.BarLayouts2[id].btn_size      = 42.8;
			nUI.BarLayouts2[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts2[id] );
		end
		
		-- bonus bar 2
	
		id = "nUI_Bar2";
		
		if nUI_BarLayouts[id] ~= nil then
			nUI.BarLayouts[id] = nUI.Class.ButtonBarLayout:new( nUI_BarLayouts[id] );
		else
			nUI.BarLayouts2[id]               = {};
			nUI.BarLayouts2[id].Origin        = {};
			nUI.BarLayouts2[id].name          = id;
			nUI.BarLayouts2[id].label         = "nUI Bar 2";
			nUI.BarLayouts2[id].button_src    = "nUIBar2Button";
			nUI.BarLayouts2[id].btn_type      = "NUIBAR2BUTTON";
			nUI.BarLayouts2[id].inherits      = "ActionBarButtonTemplate";
			nUI.BarLayouts2[id].Origin.anchor = "TOPRIGHT";
			nUI.BarLayouts2[id].Origin.yOfs   = -110;
			nUI.BarLayouts2[id].Origin.xOfs   = 682;
			nUI.BarLayouts2[id].rows          = 2;
			nUI.BarLayouts2[id].cols          = 5;
			nUI.BarLayouts2[id].btn_size      = 42.8;
			nUI.BarLayouts2[id]               = nUI.Class.ButtonBarLayout:new( nUI.BarLayouts2[id] );
		end	
		
		-- micro menu
	
		id = "nUI_MicroMenu";
		
		if nUI_Config.MicroMenu ~= nil then
			nUI.MicroMenuLayout = nUI.Class.ButtonBarLayout:new( nUI_Config.MicroMenu );
		else
			nUI.MicroMenuLayout               = {};
			nUI.MicroMenuLayout.Origin        = {};
			nUI.MicroMenuLayout.name          = id;
			nUI.MicroMenuLayout.label         = "Micro-menu";
			nUI.MicroMenuLayout.Origin.xOfs   = 0;
			nUI.MicroMenuLayout.Origin.yOfs   = -2;
			nUI.MicroMenuLayout.rows          = 1;
			nUI.MicroMenuLayout.cols          = 8;
			nUI.MicroMenuLayout.btn_size      = 55;
			nUI.MicroMenuLayout.btn_gap       = 8;
			nUI.MicroMenuLayout               = nUI.Class.ButtonBarLayout:new( nUI.MicroMenuLayout );
		end
		
		-- Faction Bar
		
		nUI.FactionBarLayout = nUI.Class.FactionBarLayout:new( nUI_Config.FactionBarLayout );
		
		-- XP Bar
		
		nUI.XPBarLayout = nUI.Class.XPBarLayout:new( nUI_Config.XPBarLayout );
		
	end
end

-------------------------------------------------------------------------------

function nUI:enteringWorld()

	nUI:setScale();
	nUI:hideBlizUI();
	
    -- now start laying out our custom UI
    
	if nUI.vars_loaded == true
	then

		if nUI.first_load then

			nUI.first_load = false;
			
			-- set up a region to overlay our chat frames and whisper frames on
			
			local width      = 580 * nUI.scale;
			local height     = 298 * nUI.scale;
			local xOfs       = 690 * nUI.scale;
			local yOfs       = -250 * nUI.scale;
		
			nUI.chat_frame = CreateFrame( "Frame", "nUI_ChatFrame", nUI_Dashboard );
			
			nUI.chat_frame:SetScale( 1.0 / nUI.scale );
			nUI.chat_frame:SetWidth( width );
			nUI.chat_frame:SetHeight( height );
			nUI.chat_frame:SetFrameStrata( "BACKGROUND" );
			nUI.chat_frame:SetFrameLevel( 3 );
			nUI.chat_frame:SetPoint( "BOTTOMRIGHT", nUI_Dashboard, "CENTER", -xOfs, yOfs );
			nUI.chat_frame:SetBackdrop(
				{
					bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
					edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 9, 
					insets   = {left = 0, right = 0, top = 0, bottom = 0},
				}
			);
			self.chat_frame:SetBackdropColor( 0, 0, 0, 0.85 );
		
			nUI.chat_frame.texture = nUI.chat_frame:CreateTexture();
			nUI.chat_frame.texture:SetAllPoints( nUI.chat_frame );
		
			-- set up a second region to overlay the combat log and other "general" info on
			
			nUI.info_frame = CreateFrame( "Frame", "nUI_ChatFrame", nUI_Dashboard );
			
			nUI.info_frame:SetScale( 1.0 / nUI.scale );
			nUI.info_frame:SetWidth( width );
			nUI.info_frame:SetHeight( height );
			nUI.info_frame:SetFrameStrata( "BACKGROUND" );
			nUI.info_frame:SetFrameLevel( 3 );
			nUI.info_frame:SetPoint( "BOTTOMLEFT", nUI_Dashboard, "CENTER", xOfs, yOfs );
			nUI.info_frame:SetBackdrop(
				{
					bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
					edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 9, 
					insets   = {left = 0, right = 0, top = 0, bottom = 0},
				}
			);
			nUI.info_frame:SetBackdropColor( 0, 0, 0, 0.85 );
			nUI.info_frame.texture = nUI.chat_frame:CreateTexture();
			nUI.info_frame.texture:SetAllPoints( nUI.info_frame );		

			-- compass ring
			
			nUI_Dashboard.compass_ring = CreateFrame( "Frame", "nUI_Dashboard_CompassRing", nUI_Dashboard );
			nUI_Dashboard.compass_ring:SetWidth( 512 );
			nUI_Dashboard.compass_ring:SetHeight( 512 );
			
			nUI_Dashboard.compass_ring.texture = nUI_Dashboard.compass_ring:CreateTexture();
			nUI_Dashboard.compass_ring.texture:SetAllPoints( nUI_Dashboard.compass_ring );
			nUI_Dashboard.compass_ring.texture:SetTexture( "Interface\\AddOns\\nUI\\Art\\nUI_CompassRing.blp" );
	
			-- create the hud
			
			local item_id = 0;
			
			nUI.hud_layout = nUI.Class.HUDLayout:new( nil );
			
			nUI.hud = nUI.Class.HUD:new( nUI_Dashboard, item_id, nUI.hud_layout );
			item_id = item_id + nUI.hud.rid;
	
			nUI:moveDashboard( nUI_Config.yOfs );
			nUI:moveConsole( nUI_Config.yOfs2 );
				
			if nUI_Config.hidden == "yes" then
				nUI_MasterFrame:Hide();
				nUI.is_hidden = true;
			else
				nUI_MasterFrame:Show();
				nUI.is_hidden = false;
			end
	
			-- create our action bars
			
			for i in pairs( nUI.BarLayouts ) do
				local layout             = nUI.BarLayouts[i];
				nUI_BarLayouts[i]        = nUI.Class.ButtonBarLayout:new( layout );
				local bar                = nUI.Class.ButtonBar:new( nUI_Dashboard, item_id, layout );
				item_id                  = item_id + layout.rows * layout.cols;
				nUI.Bars[layout.name]    = bar;
			end
			
			nUI.Bars["nUI_MainMenuBarArtFrame"]:selectBar();
		
			for i in pairs( nUI.BarLayouts2 ) do
				local layout             = nUI.BarLayouts2[i];
				nUI_BarLayouts[i]        = nUI.Class.ButtonBarLayout:new( layout );
				local bar                = nUI.Class.ButtonBar:new( nUI_Dashboard, item_id, layout );
				item_id                  = item_id + layout.rows * layout.cols;
				nUI.Bars[layout.name]    = bar;
			end

			-- create the micro menu

			nUI.Bars[nUI.MicroMenuLayout.name] = nUI.Class.ButtonBar:new( nUI_Console, item_id, nUI.MicroMenuLayout );
			item_id = item_id + nUI.MicroMenuLayout.rows * nUI.MicroMenuLayout.cols;
			
			-- layout the clock frame
			
			nUI_Dashboard.clock = CreateFrame( "Frame", "nUI_Clock", nUI_Console );
			nUI_Dashboard.clock:SetScale( 1.0 / nUI.scale );
			nUI_Dashboard.clock:SetWidth( 250 * nUI.scale );
			nUI_Dashboard.clock:SetHeight( 20 * nUI.scale );
			nUI_Dashboard.clock:SetFrameStrata( "BACKGROUND" );
			nUI_Dashboard.clock:SetFrameLevel( 3 );
			nUI_Dashboard.clock:SetPoint( "TOP", "nUI_MicroMenu", "BOTTOM", 0, 0 );
			
			nUI_Dashboard.clock.text = nUI_Dashboard.clock:CreateFontString( "nUI_ClockText", "OVERLAY" );
			nUI_Dashboard.clock.text:SetPoint( "CENTER", nUI_Dashboard.clock, "CENTER", 0, -1 );
			nUI_Dashboard.clock.text:SetFont( "Fonts\\\ARIALN.TTF", 24 * nUI.scale, "OUTLINE" );
			nUI_Dashboard.clock.text:SetTextColor( 1, 0.8196079, 0, 1 );
			nUI_Dashboard.clock.text:SetJustifyH( "CENTER" );
			nUI_Dashboard.clock.text:SetJustifyV( "MIDDLE" );
	
			-- hook the show/hide on the bonus action bar so we can do something about
			-- the standard action buttons. Front what I can tell, bliz just overlays
			-- the default action bar with the bonus action bar which is opaque. We
			-- don't have that luxury. We can't just use Show() and Hide() on the
			-- buttons, either, as dragging an action causes all buttons to be displayed
			-- in which case our "hidden" bar suddenly reappears. Rather, we have to hide
			-- the default action bar behind the dashboard when the bonus action bar is
			-- present and the pop it back up on top of the dashboard when the bonus bar
			-- is hidden. It's a PITA, and a hack, but gets the job done.
			
			local bar = _G["BonusActionBarFrame"];
			
			nUI.bonusbarOnShow = bar:GetScript( "OnShow" );
			nUI.bonusbarOnHide = bar:GetScript( "OnHide" );
			
			bar:SetScript( "OnShow", 
				function()
					if nUI.bonusbarOnShow then nUI.bonusbarOnShow(); end
					for i=1,12 do 
						local button = _G["ActionButton"..i];
						button:SetFrameStrata( "BACKGROUND" );
						button:SetFrameLevel( 0 );
					end
				end
			);
			
			bar:SetScript( "OnHide", 
				function()
					if nUI.bonusbarOnHide then nUI.bonusbarOnHide(); end
					for i=1,12 do 
						local button = _G["ActionButton"..i];
						button:SetFrameStrata( "LOW" );
						button:SetFrameLevel( 5 );
					end
				end
			);

			-- if the bonus bar is visible now, then we need to hide the default
			-- action buttons from the get-go
			
			if bar:IsShown() then 
				for i=1,12 do 
					local button = _G["ActionButton"..i];
					button:SetFrameStrata( "BACKGROUND" );
					button:SetFrameLevel( 0 );
				end
			end
			
			-- create our unit frames
	
			for i in pairs( nUI.UnitLayouts ) do
				nUI_UnitLayouts[i] = nUI.UnitLayouts[i];
				nUI.Units[nUI.UnitLayouts[i].unit_id] = nUI.Class.Unit:new( nUI_Dashboard, item_id, nUI.UnitLayouts[i] );
				item_id = item_id + (nUI.Units[nUI.UnitLayouts[i].unit_id].rid or 1);
			end

			-- add faction and experience bars
			
			nUI.FactionBar = nUI.Class.FactionBar:new( nUI_Dashboard, item_id, nUI.FactionBarLayout );
			nUI.XPBar      = nUI.Class.XPBar:new( nUI_Dashboard, item_id, nUI.XPBarLayout );
			
			-- attach the dashboard's show/hide to the UIParent's show/hide
			
			nUI.uiparentOnShow = UIParent:GetScript( "OnShow" );
			nUI.uiparentOnHide = UIParent:GetScript( "OnHide" );
			
			UIParent:SetScript( "OnShow",
				function()			
					nUI:showDB();
					if nUI.uiparentOnShow then nUI.uiparentOnShow(); end
				end
			);
			
			UIParent:SetScript( "OnHide",
				function()			
					nUI:hideDB();
					if nUI.uiparentOnHide then nUI.uiparentOnHide(); end
				end
			);
			
			-- if we haven't yet, then clean up the way Bliz wants it to look
			
			nUI:minimapFix();
			nUI:chatframeFix();	
			nUI:battlefieldMapFix()
			nUI:tooltipFix();
			nUI:buttonBagFix();
			
			nUI.suppress = false;
		end	
	end
	
	nUI:addonIntegration();
	
end

-------------------------------------------------------------------------------

function nUI:leavingWorld()
		
	if nUI_Config.hidden == "no" then
        nUI_MasterFrame:Show();
        nUI.is_hidden = false;
	end
end

-------------------------------------------------------------------------------

function nUI:refreshUnits()

	for i in pairs( nUI.Units ) do
		nUI.Units[i]:refreshUnit();
	end
end

-------------------------------------------------------------------------------
--                              UI Update Handling                           --
-------------------------------------------------------------------------------

function nUI:onUpdate( who, elapsed )

	nUI.Timers.player_coords = nUI.Timers.player_coords + elapsed;
	nUI.Timers.clock         = nUI.Timers.clock + elapsed;
	
	-- update map coordinates at just 4fps... should be more than adequate
	
	if nUI.Timers.player_coords > 0.25 then		
		nUI.Timers.player_coords = 0;
		nUI:updateCoords();
		
		if MouseIsOver( nUI.chat_frame ) then
			
			for i=1, 7, 1 do
							
				_G["ChatFrame"..i.."BottomButton"]:SetAlpha( 1 );
				_G["ChatFrame"..i.."DownButton"]:SetAlpha( 1 );
				_G["ChatFrame"..i.."UpButton"]:SetAlpha( 1 );
			end
			
			ChatFrameMenuButton:SetAlpha( 1 );
			
		else
					
			for i=1, 7, 1 do
							
				_G["ChatFrame"..i.."BottomButton"]:SetAlpha( 0 );
				_G["ChatFrame"..i.."DownButton"]:SetAlpha( 0 );
				_G["ChatFrame"..i.."UpButton"]:SetAlpha( 0 );
			end
			
			ChatFrameMenuButton:SetAlpha( 0 );			
			
		end
			
	end	
	
	-- update the game clock at just 1fps, again more than adequate
	
	if nUI.Timers.clock > 1 then		
		nUI.Timers.clock = 0;
		nUI:updateClock();		
	end
	
	-- check for new minimap buttons every five seconds for the first thirty seconds
	-- as a new load and then every 30 seconds thereafter... unfortunately, we have
	-- no way of knowing when mods post their minimap buttons or LOD mods add a new
	-- button.
	
	nUI.Timers.buttonbag = nUI.Timers.buttonbag + elapsed;
		
	if nUI.Timers.buttonbag > (nUI.buttonbag < 6 and 5 or 30) then
		
		nUI.buttonbag = nUI.buttonbag + 1;
		nUI.Timers.buttonbag = 0;
		nUI:buttonBagFix();
		
	end
end

-------------------------------------------------------------------------------

function nUI:updateCoords()
	
	local mapX, mapY = GetPlayerMapPosition( "player" );
	local text;
	
	mapX = math.floor( mapX * 1000) * 0.1;
	mapY = math.floor( mapY * 1000) * 0.1;
	
	if mapX == 0 and mapY == 0 then
		
		local inInstance, instanceType = IsInInstance();
		
		if not inInstance and not WorldMapFrame:IsVisible() then
			SetMapToCurrentZone();
		end
	end

	if math.floor( mapX ) == mapX then
		text = mapX..".0";
	else
		text = mapX;
	end
		
	text = text.." | ";
	
	if math.floor( mapY ) == mapY then
		text = text..mapY..".0";
	else
		text = text..mapY;
	end
	
	nUI_Dashboard.player_coords.text:SetText( text );
end

-------------------------------------------------------------------------------

function nUI:updateClock()

	local server_hour, server_minute = GetGameTime();
	local local_hour,  local_minute = tonumber( date( "%H" ) ), tonumber( date( "%M" ) );
	local server_suffix, local_suffix = "am", "am";
	local time_str;
	
	if server_hour > 12 then
		server_hour   = server_hour - 12;
		server_suffix = "pm";
	end
	
	if server_hour == 0 then
		server_hour = 12;
	end
	
	if local_hour > 12 then
		local_hour   = local_hour - 12;
		local_suffix = "pm";
	end
	
	if local_hour == 0 then
		local_hour = 12;
	end
	
	time_str = "(S)  "..server_hour..":";
	
	if server_minute < 10 then
		time_str = time_str.."0"..server_minute;
	else
		time_str = time_str..server_minute;
	end
	
	time_str = time_str..server_suffix.."  |  "..local_hour..":";
	
	if local_minute < 10 then
		time_str = time_str.."0"..local_minute;
	else
		time_str = time_str..local_minute;
	end
	
	nUI_Dashboard.clock.text:SetText( time_str..local_suffix.."  (L)" );
	
end

-------------------------------------------------------------------------------
--                          Slash command processing                         --
-------------------------------------------------------------------------------

function nUI:slashHelp()
    
	DEFAULT_CHAT_FRAME:AddMessage( "nUI Version "..nUI.version.." -- "..nUI.copyright, 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui help -- this message", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui hide -- hide the nUI interface panels", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui show -- show the nUI interface panels", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui flash -- highligh the active action bar", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui id -- show the action bar labels / layout", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui zoom {1|2} -- toggle chat window (1) or combat log (2) zoom on/off", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui yofs {n} -- move the dashboard {n} from the bottom of the screen", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui yofs2 {n} -- move the console {n} from the top of the screen", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui fontsize {n} -- sets the size of the chat/combat log font (default=12)", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui debug {n} -- set debugging message level", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui scale {n} -- set the dashboard scale to 0.5 < n < 1.5 or 0 for default", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui anim -- toggle animation on/off on unit frames", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui movers -- toggle movers on an off for quest watch, timer, durability, etc", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui stats -- report nUI CPU/memory usage stats", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui nostats -- stop CPU profiling (saves profiling overhead)", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui bagscale {n} -- set the scaling for bag sizes ( 0.5 < n < 1.5)", 1, 1, 0.50 );
	DEFAULT_CHAT_FRAME:AddMessage( "/nui rl -- reload the user interface", 1, 1, 0.50 );
    
end

-------------------------------------------------------------------------------

function nUI:slashCommand( msg )

	local command, arg1, arg2 = strsplit(" ", msg)
	command = string.lower( command or "help" )

	if(     command == "rl" )       then ReloadUI();
	elseif( command == "help" )     then nUI:slashHelp();
	elseif( command == "hide" )     then nUI:hideDB();
	elseif( command == "show" )     then nUI:showDB();
	elseif( command == "debug" )    then nUI:setDebug( arg1 );
	elseif( command == "bagscale" ) then nUI:setBagScale( arg1 );
	elseif( command == "flash" )    then nUI.Bars["nUI_MainMenuBarArtFrame"].active_bar:flashBar( 1, 0, 0, 1 ); 
	elseif( command == "id" )       then nUI:idBars();
	elseif( command == "yofs2" )	then nUI:moveConsole( tonumber( arg1 or 0 ) or 0 );
	elseif( command == "yofs" )		then nUI:moveDashboard( tonumber( arg1 or 0 ) or 0 );
	elseif( command == "bindings" ) then nUI:showBindings();
	elseif( command == "fontsize" ) then nUI:setChatFontSize( arg1 );
	elseif( command == "zoom" )     then nUI:toggleChatZoom( arg1 );
	elseif( command == "anim" )     then nUI:toggleAnim();
	elseif( command == "stats" )    then nUI:reportAddonStats();
	elseif( command == "movers" )	then nUI:toggleMovers();
	elseif( command == "nostats" )  then nUI:stopAddonStats();
	elseif( command == "scale" )    then nUI:changeScale( arg1 );
	elseif( command == "" )         then nUI:slashHelp();
	else DEFAULT_CHAT_FRAME:AddMessage( "nUI: unknown slash command ["..command.."] -- try \"/nui help\"", 0.75, 0.75, 0.75 );
	end
end

-------------------------------------------------------------------------------

function nUI:changeScale( scale )
	
	nUI_Config.scale = tonumber( scale );
	
	if not nUI_Config.scale or nUI_Config.scale == 0 then 
		nUI_Config.scale = nil;
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: Dashboard scale reset to default", 1, 0.83, 0 );
	else
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: Dashboard scale set to |cFF00FFFF"..nUI_Config.scale.."|r", 1, 0.83, 0 );
	end
		
	nUI:setScale();
	
end

-------------------------------------------------------------------------------

function nUI:stopAddonStats()
	
	if not nUI.profiling then
		
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: CPU profiling was already diabled", 1, 0.83, 0 );
		
	else
		
		SetCVar( "scriptProfile", 0 );
		nUI.profiling = false;
	
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: AddOn CPU Profiling |cFFFF0000DISABLED|r\nType '/nui stats' again to report results", 1, 0.83, 0 );
		
	end
end

-------------------------------------------------------------------------------

function nUI:reportAddonStats()
	
	if not nUI.profiling then
		
		SetCVar( "scriptProfile", 1 ); 
		
		ResetCPUUsage(); 
		
		nUI.profile_start = GetTime();
		nUI.profiling     = true;
		
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: AddOn CPU Profiling |cFF00FF00ENABLED|r\nType '/nui stats' again to report results", 1, 0.83, 0 );
		
	else
		
		UpdateAddOnMemoryUsage();
		UpdateAddOnCPUUsage();
		
		nUI.cpu = GetAddOnCPUUsage( "nUI" ) / (GetTime() - nUI.profile_start) * 0.1;
		nUI.mem = GetAddOnMemoryUsage( "nUI" );
		
		DEFAULT_CHAT_FRAME:AddMessage( ("nUI Usage Stats:\nCPU = |cFF00FFFF%0.1f%%|r (as %% of total elapsed CPU time)\nMemory = |cFF00FFFF%0.1fk|r bytes"):format( nUI.cpu, nUI.mem ), 1, 0.83, 0 );
	end
end

-------------------------------------------------------------------------------

function nUI:moveDashboard( yOfs )
	
	nUI_Config.yOfs = yOfs or 0;
	
	nUI_Dashboard:SetPoint( "BOTTOM", nUI_MasterFrame, "BOTTOM", 0, (yOfs or 0) / nUI.scale );
	
end

-------------------------------------------------------------------------------

function nUI:moveConsole( yOfs )
	
	nUI_Config.yOfs2 = (yOfs or 0);
	
	nUI_Console:SetPoint( "TOP", nUI_MasterFrame, "TOP", 0, -(yOfs or 0) / nUI.scale );
	
end

-------------------------------------------------------------------------------

function nUI:setBagScale( arg1 )
	
	nUI_Config.bag_scale = tonumber( arg1 );

	DEFAULT_CHAT_FRAME:AddMessage( "nUI: bag scale set to |cFF00FF00"..nUI_Config.bag_scale.."|r", 0.75, 0.75, 0.75 );
	
	nUI_manageBags();
	
end

-------------------------------------------------------------------------------

function nUI:toggleAnim()
	
	if nUI_Config.unit_anim ~= "yes" then
		if not nUI.suppress then
			DEFAULT_CHAT_FRAME:AddMessage( "nUI: animation is |cFF00FF00ENABLED|r", 0.75, 0.75, 0.75 );
		end
		nUI_Config.unit_anim = "yes";
	else
		if not nUI.suppress then
			DEFAULT_CHAT_FRAME:AddMessage( "nUI: animation is |cFFFF0000DISABLED|r", 0.75, 0.75, 0.75 );
		end
		nUI_Config.unit_anim = "no";
	end

	for i in pairs( nUI.Units ) do
		
		nUI.Units[i]:enableAnim( nUI_Config.unit_anim );
		
	end
end

-------------------------------------------------------------------------------

function nUI:setChatFontSize( n )
	
	nUI_Config.chatfont_size = (n or "12");
	
	if not nUI.suppress then
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: chat font size set to |cFF00FF00"..nUI_Config.chatfont_size.."|r", 0.75, 0.75, 0.75 );
	end
	
	for i = 1, 7, 1 do
		
		_G["ChatFrame"..i]:SetFont( "Fonts\\ARIALN.TTF", tonumber( nUI_Config.chatfont_size ) * nUI.scale * 2, "OUTLINE" );	

	end
end
	
-------------------------------------------------------------------------------

function nUI:toggleChatZoom( n )
	
	local which = n or 1;
	
	if tonumber( which ) == 2 then
		if nUI.info_zoomed then

			if not nUI.suppress then
				DEFAULT_CHAT_FRAME:AddMessage( "nUI: information frame zoom |cFFFF0000DISABLED|r", 0.75, 0.75, 0.75 );
			end
			
			nUI.info_frame:SetHeight( nUI.info_frame:GetHeight() / 3 );
			nUI.info_frame.texture:SetTexture( 0, 0, 0, 0 );
			nUI.info_zoomed = false;
			
		else
			
			if not nUI.suppress then
				DEFAULT_CHAT_FRAME:AddMessage( "nUI: information frame zoom |cFF00FF00ENABLED|r", 0.75, 0.75, 0.75 );
			end
			
			nUI.info_frame:SetHeight( nUI.info_frame:GetHeight() * 3 );
			nUI.info_frame.texture:SetTexture( 0, 0, 0, 0.5 );
			nUI.info_zoomed = true;
			
		end
	else
		if nUI.chat_zoomed then
			
			if not nUI.suppress then
				DEFAULT_CHAT_FRAME:AddMessage( "nUI: chat frame zoom |cFFFF0000DISABLED|r", 0.75, 0.75, 0.75 );
			end
			
			nUI.chat_frame:SetHeight( nUI.chat_frame:GetHeight() / 3 );
			nUI.chat_frame.texture:SetTexture( 0, 0, 0, 0 );
			nUI.chat_zoomed = false;
			
		else
			
			if not nUI.suppress then
				DEFAULT_CHAT_FRAME:AddMessage( "nUI: chat frame zoom |cFF00FF00ENABLED|r", 0.75, 0.75, 0.75 );
			end
			
			nUI.chat_frame:SetHeight( nUI.chat_frame:GetHeight() * 3 );
			nUI.chat_frame.texture:SetTexture( 0, 0, 0, 0.5 );
			nUI.chat_zoomed = true;
			
		end
	end
end

-------------------------------------------------------------------------------

function nUI:setDebug( level )

	local bad_value = true;
	local n = (tonumber( level ) or -1);
	
	if n >= 0 and n <= 9 then
		
		nUI_Config.debug = level;
		bad_value        = false;

		if not nUI.suppress then
			DEFAULT_CHAT_FRAME:AddMessage( "nUI: debug level set to |cFF00FF00"..n.."|r", 0.75, 0.75, 0.75 );
		end
	end
	
	if bad_value then
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: ["..level.."] is not a valid debug level.", 1, 0.25, 0.25 );
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: Format is /nui debug {n} when {n} is 0-9", 1, 0.25, 0.25 );
	end
end

-------------------------------------------------------------------------------

function nUI:hideDB()
		
	if nUI_Config.hidden == "no" then

		-- relocate the chat frame edit box in the same manner
		
		ChatFrameEditBox:SetParent( UIParent );
		ChatFrameEditBox:SetPoint( "TOPLEFT", UIParent, "BOTTOMLEFT", 0, 50 );
					
		nUI_Config.hidden  = "yes";
		nUI.is_hidden = true;
		nUI_MasterFrame:Hide();
	end
	
end

-------------------------------------------------------------------------------

function nUI:showDB()
	
	if nUI_Config.hidden == "yes" then

		-- relocate the chat frame edit box in the same manner
		
		nUI_Config.hidden = "no";
		nUI.is_hidden     = false;
		nUI_MasterFrame:Show();
		
		ChatFrameEditBox:SetParent( nUI.chat_frame );
		ChatFrameEditBox:SetPoint( "TOPLEFT", nUI.chat_frame, "TOPLEFT", 0, 1 );
		ChatFrameEditBox:SetPoint( "TOPRIGHT", nUI.chat_frame, "TOPRIGHT", 0, 1 );
							
	end
	
end

-------------------------------------------------------------------------------

function nUI:idBars()
	
	for i in pairs( nUI.Bars ) do 
		nUI.Bars[i]:flashLabel(); 
	end
end

-------------------------------------------------------------------------------

function nUI:showBindings()
		
	DEFAULT_CHAT_FRAME:AddMessage( "---[ Key Bindings "..nUI.last_binding.." to "..min(GetNumBindings(), (nUI.last_binding+10)).." ]--------------------", 0, 1, 1 );
	
	for i = 1, 20, 1 do
		local action, key1, key2 = GetBinding( nUI.last_binding );
		DEFAULT_CHAT_FRAME:AddMessage( "["..action.."] "..(key1 or "?").." | "..(key2 or "?"), 0.75, 0.75, 0.75 );
		nUI.last_binding = nUI.last_binding+1;
		if nUI.last_binding == GetNumBindings() then break; end
	end
	
	DEFAULT_CHAT_FRAME:AddMessage( "---------------------------------------", 0, 1, 1 );
	if nUI.last_binding >= GetNumBindings() then
		nUI.last_binding = 1;
	end
			
end

-------------------------------------------------------------------------------

function nUI:autoDing( level )
	
	if level >= 10 then
		DEFAULT_CHAT_FRAME:AddMessage( "DING! nUI says gratz to "..nUI.playerName.." for hitting level |cFFFFFFFF"..level.."|r!", 0, 1, 1, 1.0 );
	end
end

-------------------------------------------------------------------------------
--                           Blizzard UI Manipulation                        --
-------------------------------------------------------------------------------

function nUI:hideBlizUI()

	ShapeshiftBarFrame:SetClampedToScreen( false );
	ShapeshiftBarFrame:SetParent( nUI_Dashboard );
	ShapeshiftBarFrame:SetScale( 1.0 / nUI.scale );
	ShapeshiftBarFrame:SetFrameStrata( "BACKGROUND" );
	ShapeshiftBarFrame:SetFrameLevel( 0 );	
	ShapeshiftBarFrame:ClearAllPoints();
	ShapeshiftBarFrame:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
	
	PetActionBarFrame:SetClampedToScreen( false );
	PetActionBarFrame:SetParent( nUI_Dashboard );
	PetActionBarFrame:SetScale( 1.0 / nUI.scale );
	PetActionBarFrame:SetFrameStrata( "BACKGROUND" );
	PetActionBarFrame:SetFrameLevel( 0 );	
	PetActionBarFrame:ClearAllPoints();
	PetActionBarFrame:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
	
	MultiBarBottomLeft:Show();
	MultiBarBottomLeft:SetClampedToScreen( false );
	MultiBarBottomLeft:SetParent( nUI_Dashboard );
	MultiBarBottomLeft:SetScale( 1.0 / nUI.scale );
	MultiBarBottomLeft:SetFrameStrata( "BACKGROUND" );
	MultiBarBottomLeft:SetFrameLevel( 0 );	
	MultiBarBottomLeft:ClearAllPoints();
	MultiBarBottomLeft:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
	
	MultiBarBottomRight:Show();
	MultiBarBottomRight:SetClampedToScreen( false );
	MultiBarBottomRight:SetParent( nUI_Dashboard );
	MultiBarBottomRight:SetScale( 1.0 / nUI.scale );
	MultiBarBottomRight:SetFrameStrata( "BACKGROUND" );
	MultiBarBottomRight:SetFrameLevel( 0 );	
	MultiBarBottomRight:ClearAllPoints();
	MultiBarBottomRight:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
	
	MultiBarLeft:Show();
	MultiBarLeft:SetClampedToScreen( false );
	MultiBarLeft:SetParent( nUI_Dashboard );
	MultiBarLeft:SetScale( 1.0 / nUI.scale );
	MultiBarLeft:SetFrameStrata( "BACKGROUND" );
	MultiBarLeft:SetFrameLevel( 0 );	
	MultiBarLeft:ClearAllPoints();
	MultiBarLeft:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
	
	MultiBarRight:Show();
	MultiBarRight:SetClampedToScreen( false );
	MultiBarRight:SetParent( nUI_Dashboard );
	MultiBarRight:SetScale( 1.0 / nUI.scale );
	MultiBarRight:SetFrameStrata( "BACKGROUND" );
	MultiBarRight:SetFrameLevel( 0 );	
	MultiBarRight:ClearAllPoints();
	MultiBarRight:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );
		
	MainMenuBar:SetClampedToScreen( false );
	MainMenuBar:SetParent( nUI_Dashboard );
	MainMenuBar:SetScale( 1.0 / nUI.scale );
	MainMenuBar:SetFrameStrata( "LOW" );
	MainMenuBar:SetFrameLevel( 1 );	
	MainMenuBar:ClearAllPoints();
	MainMenuBar:SetPoint( "TOPRIGHT", UIParent, "BOTTOMLEFT", -1000, -1000 );

	MainMenuExpBar:Hide();
	MainMenuBarMaxLevelBar:Hide();
	MainMenuBarPerformanceBarFrame:Hide();
	MainMenuBarPerformanceBarFrameButton:Hide();
	ExhaustionTick:Hide();
	
	nUI:disableUnitFrame( PlayerFrame );
	nUI:disableUnitFrame( TargetFrame );		
	nUI:disableUnitFrame( TargetofTargetFrame );		
	nUI:disableUnitFrame( PartyMemberFrame1 );
	nUI:disableUnitFrame( PartyMemberFrame2 );
	nUI:disableUnitFrame( PartyMemberFrame3 );
	nUI:disableUnitFrame( PartyMemberFrame4 );

	TemporaryEnchantFrame:SetClampedToScreen( false );
	TemporaryEnchantFrame:ClearAllPoints();
	TemporaryEnchantFrame:SetPoint( "BOTTOMLEFT", UIParent, "TOPRIGHT", 1000, 1000 );	
	TemporaryEnchantFrame:Hide();

	BuffFrame:SetClampedToScreen( false );
	BuffFrame:ClearAllPoints();
	BuffFrame:SetPoint( "BOTTOMLEFT", UIParent, "TOPRIGHT", 1000, 1000 );	
	BuffFrame:Hide();
	
	CastingBarFrame:SetScript( "OnShow", function() CastingBarFrame:Hide(); end );
	
end

-------------------------------------------------------------------------------

function nUI:disableUnitFrame( frame )
	
	UnregisterUnitWatch( frame );
	
	frame:UnregisterAllEvents();	
	frame:Hide();
	
	frame:ClearAllPoints();
	frame:SetPoint( "TOPRIGHT", UIParent, "TOPRIGHT", -1000, -1000 );

	local health = getglobal( frame:GetName().."HealthBar" );
	if health then health:UnregisterAllEvents(); end

	local mana = getglobal( frame:GetName().."ManaBar" );
	if mana then mana:UnregisterAllEvents(); end
	
end

-------------------------------------------------------------------------------

function nUI:tooltipFix()
	
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint( "BOTTOMRIGHT", nUI_Dashboard, "TOPRIGHT", -25 * nUI.scale, 200 );
	
end
	
-------------------------------------------------------------------------------

function nUI:chatframeFix()

	-- relocate the chat frame to the area we have set aside for it on the dashboard
	
	
	FCF_SetLocked( ChatFrame1, nil );
	MOVING_CHAT_FRAME = DEFAULT_CHAT_FRAME;
	
	ChatFrame1:StartMoving();
	ChatFrame1:StartSizing();
	ChatFrame1:SetParent( nUI.chat_frame );
	ChatFrame1:ClearAllPoints();
	ChatFrame1:SetPoint( "BOTTOMLEFT", nUI.chat_frame, "BOTTOMLEFT", 4, 6 );
	ChatFrame1:SetPoint( "TOPRIGHT", nUI.chat_frame, "TOPRIGHT", -5, -5 );
	ChatFrame1Tab:Hide();
	ChatFrame1:StopMovingOrSizing();
	
	UIParent_ManageFramePositions();

	MOVING_CHAT_FRAME = nil;

	-- clean up the combat frame
	
	ChatFrame_RemoveAllChannels( ChatFrame2 );
	ChatFrame_RemoveAllMessageGroups( ChatFrame2 );
	ChatFrame_ActivateCombatMessages( ChatFrame2 );

	-- set strata, etc, for all defined frames, make sure they're all docked
	
	for i=1, NUM_CHAT_WINDOWS do
		
		local frame  = _G["ChatFrame"..i];
		local bottom = _G["ChatFrame"..i.."BottomButton"];
		local down   = _G["ChatFrame"..i.."DownButton"];
		local up     = _G["ChatFrame"..i.."UpButton"];

		if i == 2 and not frame.isDocked then
			FCF_DockFrame( frame, 2, false );
		end
			
		FCF_SetLocked( frame, 1 );
		
		frame:SetFrameStrata( "BACKGROUND" );
	
		-- relocate the chat frame buttons

		bottom:ClearAllPoints();
		down:ClearAllPoints();
		up:ClearAllPoints();
		
		if i == 1 then
			frame:SetWidth( frame:GetRight() - frame:GetLeft() );
			frame:SetHeight( frame:GetTop() - frame:GetBottom() );
			bottom:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", 2, -8 );
			down:SetPoint( "BOTTOM", bottom, "TOP", 0, -6 );
			up:SetPoint( "BOTTOM", down, "TOP", 0, -6 );
		else
			frame:SetWidth( ChatFrame1:GetWidth() );
			frame:SetHeight( ChatFrame1:GetHeight() );
			bottom:SetAllPoints( ChatFrame1BottomButton );
			down:SetAllPoints( ChatFrame1DownButton );
			up:SetAllPoints( ChatFrame1UpButton );
		end
		
		-- allow scrolling of the combat log frame with the mouse wheel
		
		frame:EnableMouseWheel( true );	
		
		frame:SetScript( "OnMouseWheel", function()
			if arg1 > 0 then frame:ScrollUp();
			else frame:ScrollDown();
			end	
		end	);		
	end
	
	-- relocate the chat frame edit box in the same manner
	
	if nUI_Config.hidden ~= "yes" then
		ChatFrameEditBox:SetParent( nUI.chat_frame );
		ChatFrameEditBox:SetPoint( "TOPLEFT", nUI.chat_frame, "TOPLEFT", 0, 1 );
		ChatFrameEditBox:SetPoint( "TOPRIGHT", nUI.chat_frame, "TOPRIGHT", 0, 1 );
	else
		ChatFrameEditBox:SetParent( UIParent );
		ChatFrameEditBox:SetPoint( "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 10, 10 );
	end
	
	ChatFrameMenuButton:ClearAllPoints();
	ChatFrameMenuButton:SetPoint( "BOTTOM", ChatFrame1UpButton, "TOP", 0, -6 );
	ChatFrameMenuButton:Hide(); -- be gone pest
	
	nUI:setChatFontSize( nUI_Config.chatfont_size );
	
	-- Update all the anchors
	
	UIParent_ManageFramePositions();
	
end

-------------------------------------------------------------------------------

function nUI:minimapFix()

	MinimapCluster:SetParent( nUI_Dashboard );
	MinimapCluster:SetScale( 1.9 );
	MinimapCluster:SetFrameStrata( "BACKGROUND" );
	MinimapCluster:SetFrameLevel( 2 );
	MinimapCluster:ClearAllPoints();
	MinimapCluster:SetPoint( "CENTER", nUI_Dashboard, "CENTER", -20 * nUI.scale, -120 * nUI.scale );
	
	MinimapBorder:Hide();
	MinimapBorderTop:Hide();
	MinimapToggleButton:Hide();
	MinimapZoneTextButton:Hide();
	
	Minimap:SetFrameStrata( "BACKGROUND" );
	
	MinimapZoomIn:SetParent( Minimap );
	MinimapZoomIn:SetScale( 0.85 );
	MinimapZoomIn:SetFrameStrata( "BACKGROUND" );
	MinimapZoomIn:SetPoint( "CENTER", Minimap, "CENTER", 49 / 0.85, -63 / 0.85 );

	MinimapZoomOut:SetParent( Minimap );
	MinimapZoomOut:SetScale( 0.85 );
	MinimapZoomOut:SetFrameStrata( "BACKGROUND" );
	MinimapZoomOut:SetPoint( "CENTER", Minimap, "CENTER", -50 / 0.85, -63 / 0.85 );
	
	MiniMapTracking:ClearAllPoints();
	MiniMapTracking:SetParent( Minimap );
	MiniMapTracking:SetFrameStrata( "BACKGROUND" );
	MiniMapTracking:SetScale( 0.7 );
	MiniMapTracking:SetPoint( "CENTER", Minimap, "CENTER", -90 / 0.85, -48 / 0.85 );
	
	MiniMapWorldMapButton:ClearAllPoints();
	MiniMapWorldMapButton:SetScale( 0.85 );
	MiniMapWorldMapButton:SetParent( Minimap );
	MiniMapWorldMapButton:SetPoint( "CENTER", Minimap, "CENTER", 69 / 0.85, -42 / 0.85 );
	
	MinimapNorthTag:SetHeight( MinimapNorthTag:GetHeight() * 1.2 );
	MinimapNorthTag:SetWidth(MinimapNorthTag:GetWidth() * 1.2 );
	
	MiniMapCompassRing:ClearAllPoints();
	MiniMapCompassRing:SetParent( Minimap );
	MiniMapCompassRing:SetFrameStrata( "BACKGROUND" );
	MiniMapCompassRing:SetPoint( "CENTER", Minimap, "CENTER", 0, 0 );

	MiniMapMailFrame:SetScale( 0.85 );
	MiniMapMailFrame:ClearAllPoints();
	MiniMapMailFrame:SetFrameStrata( "BACKGROUND" );
	MiniMapMailFrame:SetPoint( "CENTER", Minimap, "CENTER", 0, -66 / 0.85 );

	MiniMapBattlefieldFrame:SetScale( 0.85 );
	MiniMapBattlefieldFrame:ClearAllPoints();
	MiniMapBattlefieldFrame:SetFrameStrata( "BACKGROUND" );
	MiniMapBattlefieldFrame:SetPoint( "CENTER", Minimap, "CENTER", 49 / 0.85, 64 / 0.85 );

	MiniMapMeetingStoneFrame:SetScale( 0.85 );
	MiniMapMeetingStoneFrame:ClearAllPoints();
	MiniMapMeetingStoneFrame:SetFrameStrata( "BACKGROUND" );
	MiniMapMeetingStoneFrame:SetPoint( "CENTER", Minimap, "CENTER", -52 / 0.85, 64 / 0.85 );
	
	MiniMapVoiceChatFrame:SetScale( 0.85 );
	MiniMapVoiceChatFrame:ClearAllPoints();
	MiniMapVoiceChatFrame:SetFrameStrata( "BACKGROUND" );
	MiniMapVoiceChatFrame:SetPoint( "CENTER", Minimap, "CENTER", -68 / 0.85, 43 / 0.85 );
	
	MiniMapRecordingButton:SetScale( 0.85 );
	MiniMapRecordingButton:ClearAllPoints();
	MiniMapRecordingButton:SetFrameStrata( "BACKGROUND" );
	MiniMapRecordingButton:SetPoint( "CENTER", Minimap, "CENTER", 68 / 0.85, 43 / 0.85 );

	GameTimeFrame:Hide();
	
	Minimap:EnableMouseWheel( true );	
	
	Minimap:SetScript( "OnMouseWheel", function()
		if arg1 > 0 then Minimap_ZoomIn();
		else Minimap_ZoomOut();
		end	
	end	);

	nUI_Dashboard.compass_ring:SetPoint( "CENTER", Minimap, "CENTER", 0, 0 );
	
	nUI_Dashboard.player_coords = CreateFrame( "Frame", "nUI_PlayerCoords", Minimap );
	nUI_Dashboard.player_coords:SetWidth( MinimapZoneText:GetWidth() );
	nUI_Dashboard.player_coords:SetHeight( MinimapZoneText:GetHeight() );
	nUI_Dashboard.player_coords:SetPoint( "BOTTOM", Minimap, "TOP", 0, 8 );
	
	nUI_Dashboard.player_coords.text = nUI_Dashboard.player_coords:CreateFontString( "nUI_PlayerCoordsText", "OVERLAY" );
	nUI_Dashboard.player_coords.text:SetAllPoints( nUI_Dashboard.player_coords );
	nUI_Dashboard.player_coords.text:SetFont( "Fonts\\\ARIALN.TTF", 10.25, "OUTLINE" );
	nUI_Dashboard.player_coords.text:SetTextColor( 1, 0.8196079, 0, 1 );
	nUI_Dashboard.player_coords.text:SetJustifyH( "CENTER" );
	nUI_Dashboard.player_coords.text:SetJustifyV( "MIDDLE" );
	
	nUI_Dashboard.location_backdrop = CreateFrame( "Frame", "nUI_Dashboard_LocBackdrop", Minimap );
	nUI_Dashboard.location_backdrop:SetWidth( 512 );
	nUI_Dashboard.location_backdrop:SetHeight( 128 );
	nUI_Dashboard.location_backdrop:SetScale( 0.5 );
	nUI_Dashboard.location_backdrop:SetPoint( "CENTER", nUI_Dashboard.player_coords, "TOP", 0, -5 );
	nUI_Dashboard.location_backdrop:SetFrameLevel( 2 );
	nUI_Dashboard.location_backdrop:SetAlpha( 0.6 );
	
	nUI_Dashboard.location_backdrop.texture = nUI_Dashboard.location_backdrop:CreateTexture();
	nUI_Dashboard.location_backdrop.texture:SetAllPoints( nUI_Dashboard.location_backdrop );
	nUI_Dashboard.location_backdrop.texture:SetTexture( "Interface\\AddOns\\nUI\\Art\\nUI_LocationBackdrop.blp" );

	MinimapZoneText:ClearAllPoints();
	MinimapZoneText:SetParent( Minimap );
	MinimapZoneText:SetPoint( "BOTTOM", nUI_Dashboard.player_coords, "TOP", 0, -2 );
		
	-- create movers for the various UI frames that interest us
	
	nUI:initMovers();
	
end

-------------------------------------------------------------------------------
--                         Hooked Bliz Functions                             --
-------------------------------------------------------------------------------

function nUI_manageBags()

	local screen_height = GetScreenHeight();
	local view_height   = screen_height - 440 * nUI.scale;
	local bag_scale     = nUI_Config.bag_scale;
	
	repeat 
		
		local left_edge = -GetScreenWidth();
		local xOfs      = left_edge * 0.01;		
		local yOfs      = (screen_height - view_height)  / UIParent:GetScale();
		local bag_width = CONTAINER_WIDTH * bag_scale;

		if BankFrame:IsShown() then
			left_edge = left_edge + BankFrame:GetRight();
		end
		
		for i, bag_name in ipairs( ContainerFrame1.bags ) do
			
			local frame = getglobal( bag_name );
			local bag_height = frame:GetHeight() * bag_scale;
			
			frame:SetScale( bag_scale );
			
			if (yOfs + bag_height) > screen_height then			
				xOfs = xOfs - bag_width;
				yOfs = (screen_height - view_height) / UIParent:GetScale();
			end
			
			frame:ClearAllPoints();
			frame:SetPoint( "BOTTOMRIGHT", nUI_Dashboard, "BOTTOMRIGHT", xOfs / bag_scale, yOfs / bag_scale );
			
			yOfs = yOfs + bag_height + VISIBLE_CONTAINER_SPACING;
			
		end	
		
		bag_scale = bag_scale - 0.01;

	until xOfs >= left_edge;	
end

-------------------------------------------------------------------------------
-- Used to hook the Bliz ActionButton_OnUpdate and ActionButton_UpdateUsable --
-- in order to provide OOM and OOR coloring along with cooldown counters on  --
-- action buttons                                                            --
-------------------------------------------------------------------------------

function nUI_Button_OnUpdate( elapsed )

	if this.nUI_class then 
		if this.nUI_class:onActionUpdate( elapsed ) then
			ActionButton_UpdateUsable();
		end
	end	
end

-------------------------------------------------------------------------------

function nUI_Button_UpdateUsable()

	if this.nUI_class then 
		this.nUI_class:onActionUsable(); 
	end
end

-------------------------------------------------------------------------------

function nUI_Button_Update()

	if this.nUI_class then 
		this:SetNormalTexture( "" );
	end
end

-------------------------------------------------------------------------------

function nUI_Button_CooldownTimer( cooldown, start, duration, enable )

	if this.nUI_class then 
		this.nUI_class:startCooldown( cooldown, start, duration, enable ); 
	end
end

-------------------------------------------------------------------------------
--                      Integration with supported AddOns                    --
-------------------------------------------------------------------------------

function nUI:addonIntegration()

	-- detect which integrated AddOns are present
	
	nUI.AddOn.mi2     = IsAddOnLoaded( "MobInfo2" );
	nUI.AddOn.recount = IsAddOnLoaded( "Recount" );	
	nUI.AddOn.omen2   = IsAddOnLoaded( "Omen2" );	
	nUI.AddOn.ktm     = IsAddOnLoaded( "KLHThreatMeter" );	

	-- create a button for managing info frames
	
	nUI.info_frame.button = CreateFrame( "Button", "nUI_InfoFrame_Button", nUI.info_frame );
	nUI.info_frame.button:SetNormalTexture( "Interface\\AddOns\\nUI\\Art\\nUI_Info.blp" );
	nUI.info_frame.button:SetHighlightTexture( "Interface\\AddOns\\nUI\\Art\\nUI_InfoHighlight.blp" );
	nUI.info_frame.button:SetFrameLevel( 4 );
	nUI.info_frame.button:LockHighlight();
	nUI.info_frame.button:SetWidth( 75 * nUI.scale );
	nUI.info_frame.button:SetHeight( 75 * nUI.scale );
	nUI.info_frame.button:SetPoint( "BOTTOM", nUI.info_frame, "TOP", -85 * nUI.scale, 5 * nUI.scale );
	nUI.info_frame.button:SetFont( "Fonts\\FRIZQT__.TTF", 16 * nUI.scale, "OUTLINE" );
	nUI.info_frame.button:SetText( "BF" );
	nUI.info_frame.button:SetTextColor( 1, 1, 1, 1 );
	
	nUI.info_frame.button.text      = nUI.info_frame.button:GetFontString();
	nUI.info_frame.button.normal    = nUI.info_frame.button:GetNormalTexture();
	nUI.info_frame.button.highlight = nUI.info_frame.button:GetHighlightTexture();

	nUI.info_frame.button.text:ClearAllPoints();
	nUI.info_frame.button.text:SetPoint( "CENTER", nUI.info_frame.button, "CENTER", 1, 3 );
	nUI.info_frame.button.highlight:SetVertexColor( 0, 0, 0, 0 );
	nUI.info_frame.button.normal:SetAlpha( 0.75 );
	
	nUI.info_frame.button:SetScript( "OnClick", function() nUI:nextInfoMode(); end );

	nUI.info_frame.num_modes = 0;
	nUI.info_frame.modeName = {};
	nUI.info_frame.showMode = {};
	nUI.info_frame.hideMode = {};

	nUI:addInfoMode( "Map", function() nUI:bmmShow(); end, function() nUI:bmmHide(); end );
	
	-- create the combat stats management frame if any of those mods are present

	if nUI.AddOn.recount or nUI.AddOn.omen2 or nUI.AddOn.klh then
		
		nUI.info_frame.stats = CreateFrame( "Frame", "nUI_InfoFrame_Stats", nUI.info_frame );
		nUI.info_frame.stats:SetHeight( nUI.info_frame:GetHeight() - 2 );
		nUI.info_frame.stats:SetWidth( BattlefieldMinimap:GetWidth() * nUI.info_frame:GetHeight() / BattlefieldMinimap:GetHeight() );
		nUI.info_frame.stats:SetPoint( "TOPLEFT", nUI.info_frame, "TOPLEFT", 1, -1 );
		nUI.info_frame.stats:SetFrameStrata( nUI.info_frame:GetFrameStrata() );
		nUI.info_frame.stats:SetFrameLevel( nUI.info_frame:GetFrameLevel()+1 );
		
	end
	
	-- initialize their management
	
	if nUI.AddOn.mi2 then nUI:initMI2(); end
	if nUI.AddOn.recount then nUI:initRecount(); end
	if nUI.AddOn.ktm then nUI:initKTM(); end

	-- add the raid info frame
	
	nUI:createRaidInfo();
	
	-- select the default info frame
	
	nUI:selectInfoMode( nUI.info_frame, nUI_Config.activeInfoMode or 1 );
	
end

-------------------------------------------------------------------------------

function nUI:addInfoMode( name, show_func, hide_func )
	
	local ndx = nUI.info_frame.num_modes+1;
	nUI.info_frame.num_modes = ndx;
	
	nUI.info_frame.modeName[ndx] = name;
	nUI.info_frame.showMode[ndx] = show_func;
	nUI.info_frame.hideMode[ndx] = hide_func;
	
end

-------------------------------------------------------------------------------

function nUI:nextInfoMode()
	
	local info_frame = nUI.info_frame;
	local ndx;
	
	if nUI_Config.activeInfoMode >= info_frame.num_modes then
		ndx = 1;
	else
		ndx = nUI_Config.activeInfoMode+1;
	end
	
	nUI:selectInfoMode( info_frame, ndx );

end

-------------------------------------------------------------------------------

function nUI:selectInfoMode( info_frame, ndx )
	
	nUI_Config.activeInfoMode = ndx;
	
	info_frame.button:SetText( info_frame.modeName[ndx] );

	if info_frame.modeName[ndx] == "Raid" then
		info_frame.buttonbag:Hide();
	else
		info_frame.buttonbag:Show();
	end
	
	for i=1,info_frame.num_modes do
		
		if i == ndx then 
			info_frame.showMode[i]();
		
			info_frame.button:SetScript( "OnEnter",
				function()
					GameTooltip:SetOwner( info_frame.button, "ANCHOR_TOPLEFT", 0, 0 );
					GameTooltip:SetText( "Click to change nUI\ninformation display modes", 1, 1, 1 );
					GameTooltip:Show();
				end
			);
			info_frame.button:SetScript( "OnLeave",
				function()
					GameTooltip:Hide();
				end
			);
			
		else info_frame.hideMode[i]();
		end
	end
end
